@@ -1,24 +1,25 @@
 /*
- *  libzvbi - BCD arithmetic for Teletext page numbers
+ *  libzvbi -- BCD arithmetic for Teletext page numbers
  *
  *  Copyright (C) 2001, 2002 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: bcd.h,v 1.12 2003/11/20 20:04:07 mschimek Exp $ */
+/* $Id: bcd.h,v 1.19 2008/02/21 07:18:52 mschimek Exp $ */
 
 #ifndef BCD_H
 #define BCD_H
@@ -27,7 +28,7 @@
 
 /**
  * @addtogroup BCD BCD arithmetic for Teletext page numbers
- * @ingroup Service
+ * @ingroup HiDec
  *
  * Teletext page numbers are expressed as packed binary coded decimal
  * numbers in range 0x100 to 0x8FF. The bcd format encodes one decimal
@@ -39,7 +40,7 @@
 /* Public */
 
 /**
- * @ingroup Service
+ * @ingroup HiDec
  * 
  * Teletext or Closed Caption page number. For Teletext pages
  * this is a packed bcd number in range 0x100 ... 0x8FF. Page
@@ -74,7 +75,7 @@
 typedef int vbi_pgno;
 
 /**
- * @ingroup Service
+ * @ingroup HiDec
  *
  * This is the subpage number only applicable to Teletext pages,
  * a packed bcd number in range 0x00 ... 0x99. On special 'clock' pages
@@ -85,11 +86,11 @@ typedef int vbi_pgno;
 typedef int vbi_subno;
 
 /**
- * @ingroup Service
+ * @ingroup HiDec
  */
 #define VBI_ANY_SUBNO 0x3F7F
 /**
- * @ingroup Service
+ * @ingroup HiDec
  */
 #define VBI_NO_SUBNO 0x3F7F
 
@@ -104,12 +105,18 @@ typedef int vbi_subno;
  * @return
  * BCD number.
  */
-static_inline unsigned int
+_vbi_inline unsigned int
 vbi_dec2bcd(unsigned int dec)
 {
 	return (dec % 10) + ((dec / 10) % 10) * 16 + ((dec / 100) % 10) * 256;
 }
 
+/**
+ * @ingroup BCD
+ * @since 0.2.28
+ */
+#define vbi_bin2bcd(n) vbi_dec2bcd(n)
+
 /**
  * @ingroup BCD
  * @param bcd BCD number.
@@ -122,12 +129,18 @@ vbi_dec2bcd(unsigned int dec)
  * Decimal number. The result is undefined when the bcd number contains
  * hex digits 0xA ... 0xF.
  **/
-static_inline unsigned int
+_vbi_inline unsigned int
 vbi_bcd2dec(unsigned int bcd)
 {
 	return (bcd & 15) + ((bcd >> 4) & 15) * 10 + ((bcd >> 8) & 15) * 100;
 }
 
+/**
+ * @ingroup BCD
+ * @since 0.2.28
+ */
+#define vbi_bcd2bin(n) vbi_bcd2dec(n)
+
 /**
  * @ingroup BCD
  * @param a BCD number.
@@ -142,7 +155,7 @@ vbi_bcd2dec(unsigned int bcd)
  * Packed bcd number. The result is undefined when any of the arguments
  * contain hex digits 0xA ... 0xF.
  */
-static_inline unsigned int
+_vbi_inline unsigned int
 vbi_add_bcd(unsigned int a, unsigned int b)
 {
 	unsigned int t;
@@ -166,7 +179,7 @@ vbi_add_bcd(unsigned int a, unsigned int b)
  * @return
  * @c FALSE if @a bcd contains hex digits 0xA ... 0xF.
  */
-static_inline vbi_bool
+_vbi_inline vbi_bool
 vbi_is_bcd(unsigned int bcd)
 {
 	static const unsigned int x = 0x06666666;
@@ -174,6 +187,37 @@ vbi_is_bcd(unsigned int bcd)
 	return (((bcd + x) ^ (bcd ^ x)) & 0x11111110) == 0;
 }
 
+/**
+ * @ingroup BCD
+ * @param bcd Unsigned BCD number.
+ * @param maximum Unsigned maximum value.
+ *
+ * Compares an unsigned packed bcd number digit-wise against a maximum
+ * value, for example 0x295959. @a maximum can contain digits 0x0
+ * ... 0xF.
+ *
+ * @return
+ * @c TRUE if any digit of @a bcd is greater than the
+ * corresponding digit of @a maximum.
+ *
+ * @since 0.2.28
+ */
+_vbi_inline vbi_bool
+vbi_bcd_digits_greater		(unsigned int		bcd,
+				 unsigned int		maximum)
+{
+	maximum ^= ~0;
+
+	return 0 != (((bcd + maximum) ^ bcd ^ maximum) & 0x11111110);
+}
+
 /* Private */
 
 #endif /* BCD_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
