@@ -1,32 +1,31 @@
 /*
- *  libzvbi - Events
+ *  libzvbi -- Events
  *
  *  Copyright (C) 2000, 2001, 2002 Michael H. Schimek
  *
- *  Based on code from AleVT 1.5.1
- *  Copyright (C) 1998,1999 Edgar Toernig (froese@gmx.de)
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: event.h,v 1.5 2002/10/22 04:41:18 mschimek Exp $ */
+/* $Id: event.h,v 1.16 2009/12/14 23:43:29 mschimek Exp $ */
 
 #ifndef EVENT_H
 #define EVENT_H
 
 #include "bcd.h"
+#include "pdc.h"
 
 #ifndef VBI_DECODER
 #define VBI_DECODER
@@ -39,7 +38,7 @@ typedef struct vbi_decoder vbi_decoder;
 
 /**
  * @addtogroup Event Events
- * @ingroup Service
+ * @ingroup HiDec
  *
  * Typically the transmission of VBI data like a Teletext or Closed
  * Caption page spans several VBI lines or even video frames. So internally
@@ -68,24 +67,67 @@ typedef unsigned int vbi_nuid;
  * @ingroup Event
  * @brief Network description.
  *
- * All strings are ISO 8859-1, local language, and @c NUL terminated.
- * Prepare for empty strings. Read only.
+ * All strings are ISO 8859-1 encoded (yes that's stupid, sorry)
+ * and @c NUL terminated. Prepare for empty strings. Read only.
  */
 typedef struct {
 	vbi_nuid		nuid;
 
-	signed char		name[64];		/* descriptive name */
-	signed char		call[40];		/* network call letters (XDS) */
+	/**
+	 * Name of the network from XDS or from a table lookup of
+	 * CNIs in Teletext packet 8/30 or VPS. 
+	 */ 
+	signed char		name[64];
 
-	int			tape_delay;		/* tape delay, minutes (XDS) */
+	/**
+	 * Network call letters, from XDS. Empty if unknown or
+	 * not applicable.
+	 */
+	signed char		call[40];
 
-	/* Private */
+	/**
+	 * Tape delay in minutes, from XDS. Zero if unknown or
+	 * not applicable.
+	 **/
+	int			tape_delay;
 
+	/**
+	 * The European Broadcasting Union (EBU) maintains several tables
+	 * of Country and Network Identification (CNI) codes. CNIs of type
+	 * VPS, 8/30/1 and 8/30/2 can be used to identify networks during
+	 * a channel scan.
+	 *
+	 * This field contains the CNI of the network found in a VPS
+	 * packet. It can be zero if unknown or CNI's are not applicable.
+	 * Note VPS has room for only 4 lsb of the country code (0xCNN).
+	 *
+	 * For example ZDF: 0xDC2.
+	 */
 	int			cni_vps;
-	int			cni_8301;
-	int			cni_8302;
-	int			cni_x26;
 
+	/**
+	 * CNI of the network from Teletext packet 8/30 format 1,
+	 * zero if unknown or not applicable. The country code is
+	 * stored in the MSB, the network code in the LSB (0xCCNN).
+	 * Note these CNIs may use different country and network codes
+	 * than the PDC (VPS, 8/30/2) CNIs.
+	 *
+	 * For example BBC1: 0x447F, ZDF: 0x4902.
+	 */
+	int			cni_8301;
+
+	/**
+	 * CNI of the network from Teletext packet 8/30 format 2,
+	 * zero if unknown or not applicable. The country code is
+	 * stored in the MSB, the network code in the LSB (0xCCNN).
+	 *
+	 * For example BBC1: 0x2C7F, ZDF: 0x1DC2.
+	 */
+	int			cni_8302;
+
+	int			reserved;
+
+	/** Private. */
 	int			cycle;
 } vbi_network;
 
@@ -497,15 +539,6 @@ typedef struct vbi_program_info {
 extern void		vbi_reset_prog_info(vbi_program_info *pi);
 /** @} */
 
-/* Future */
-typedef struct pfc_block {
-	vbi_pgno		pgno;
-	int			stream;
-	int			application_id;
-	int			block_size;
-	uint8_t			block[2048];
-} pfc_block;
-
 /**
  * @ingroup Event
  * @name Event types.
@@ -573,7 +606,6 @@ typedef struct pfc_block {
  * <table>
  * <tr><td>VPS (DE/AT/CH only):</td><td>0.08 s</td></tr>
  * <tr><td>Teletext PDC, 8/30:</td><td>2 s</td></tr>
- * <tr><td>Teletext X/26:</td><td>unknown</td></tr>
  * <tr><td>XDS (US only):</td><td>unknown, between 0.1x to 10x seconds</td></tr>
  * </table>
  *
@@ -614,8 +646,84 @@ typedef struct pfc_block {
  * XXX Change to get_prog_info. network ditto?
  */
 #define	VBI_EVENT_PROG_INFO	0x0080
+/**
+ * Like @a VBI_EVENT_NETWORK, but this event will also be sent
+ * when the decoder cannot determine a network name.
+ *
+ * @since 0.2.20
+ */
+#define	VBI_EVENT_NETWORK_ID	0x0100
+/**
+ * A new local time has been received. ev.local_time points to a
+ * vbi_local_time structure with details.
+ */
+#define VBI_EVENT_LOCAL_TIME	0x0400
+/**
+ * A new Program ID (VPS or PDC) has been received. ev.prog_id points
+ * to a vbi_program_id structure with details.
+ */
+#define VBI_EVENT_PROG_ID	0x0800
 /** @} */
 
+/**
+ * Specifies if daylight-saving time is in effect in the time zone of
+ * the intended audience of the network.
+ */
+typedef enum {
+	/** The network does not provide any DST information. */
+	VBI_DST_UNKNOWN = 0,
+
+	/**
+	 * A DST offset (+0 or +1 hour) has been added to the time
+	 * zone offset.
+	 */
+	VBI_DST_INCLUDED,
+
+	/** Daylight-saving time is not in effect. */
+	VBI_DST_INACTIVE,
+
+	/**
+	 * Daylight-saving time is in effect, and +1 hour has been
+	 * added to the time zone offset.
+	 */
+	VBI_DST_ACTIVE
+} vbi_dst_state;
+
+typedef struct {
+	/** The current time in the UTC zone. */
+	time_t			time;
+
+	/**
+	 * The offset of the time zone of the intended audience of the
+	 * network in seconds east of UTC. For example the EST zone is
+	 * offset by -18000, GMT by 0, and CET by +3600 seconds. An
+	 * additional +3600 second daylight-saving time offset may
+	 * have been added as specified by @a dst_state, giving for
+	 * example an EDT offset of -14400 seconds.
+	 */
+	int			seconds_east;
+
+	/** If FALSE, the network does not provide a time zone offset. */
+	vbi_bool		seconds_east_valid;
+
+	/**
+	 * Whether daylight-saving time is currently in effect in the
+	 * time zone of the intended audience of the network.
+	 */
+	vbi_dst_state		dst_state;
+} vbi_local_time;
+
+/* Experimental CC608 decoder. */
+#define _VBI_EVENT_CC608 0x1000
+#define _VBI_EVENT_CC608_STREAM 0x2000
+struct _vbi_event_cc608_page;
+struct _vbi_event_cc608_stream;
+
+/**
+ * @example examples/network.c
+ * Network identification example.
+ */
+
 #include <inttypes.h>
 
 /**
@@ -644,6 +752,12 @@ typedef struct vbi_event {
                 vbi_link *		trigger;
                 vbi_aspect_ratio	aspect;
 		vbi_program_info *	prog_info;
+		vbi_local_time *	local_time;
+		vbi_program_id *	prog_id;
+
+		/* Experimental. */
+		struct _vbi_event_cc608_page *		_cc608;
+		struct _vbi_event_cc608_stream *	_cc608_stream;
 	}			ev;
 } vbi_event;
 
@@ -671,3 +785,10 @@ extern void		vbi_event_handler_unregister(vbi_decoder *vbi,
 extern void		vbi_send_event(vbi_decoder *vbi, vbi_event *ev);
 
 #endif /* EVENT_H */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
