@@ -1,7 +1,7 @@
 /*
  *  libzvbi - VTX export function
  *
- *  Copyright (C) 2001 Michael H. Schimek
+ *  Copyright (C) 2001, 2007 Michael H. Schimek
  *
  *  Based on code from AleVT 1.5.1
  *  Copyright (C) 1998, 1999 Edgar Toernig <froese@gmx.de>
@@ -22,10 +22,16 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-/* $Id: exp-vtx.c,v 1.7 2003/10/16 18:16:11 mschimek Exp $ */
+/* $Id: exp-vtx.c,v 1.12 2008/02/19 00:35:15 mschimek Exp $ */
+
+/* Disabled for now because this code is licensed under GPLv2+ and
+   cannot be linked with the rest of libzvbi which is licensed under
+   LGPLv2+ since version 0.2.28. */
+
+#if 0
 
 /*
  *  VTX is the file format used by VideoteXt. It stores Teletext pages in
@@ -38,7 +44,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "../config.h"
+#  include "config.h"
 #endif
 
 #include <stdio.h>
@@ -68,14 +74,16 @@ struct header {
  */
 
 static vbi_bool
-export(vbi_export *e, FILE *fp, vbi_page *pg)
+export(vbi_export *e, vbi_page *pg)
 {
 	vt_page page, *vtp;
-	struct header h;
+	struct header *h;
+	size_t needed;
 
 	if (pg->pgno < 0x100 || pg->pgno > 0x8FF) {
 		/* TRANSLATORS: Not Closed Caption pages. */
-		vbi_export_error_printf(e, _("Can only export Teletext pages."));
+		vbi_export_error_printf
+			(e, _("Can only export Teletext pages."));
 		return FALSE;
 	}
 
@@ -93,36 +101,38 @@ export(vbi_export *e, FILE *fp, vbi_page *pg)
 
 	if (page.function != PAGE_FUNCTION_UNKNOWN
 	    && page.function != PAGE_FUNCTION_LOP) {
-		vbi_export_error_printf(e, _("Cannot export this page, not displayable."));
+		vbi_export_error_printf
+			(e, _("Cannot export this page, not displayable."));
 		return FALSE;
 	}
 
-	memcpy(h.signature, "VTXV4", 5);
+	needed = sizeof (*h);
+	if (VBI_EXPORT_TARGET_ALLOC == e->target)
+		needed += 40 * 24;
 
-	h.pagenum_l = page.pgno & 0xFF;
-	h.pagenum_h = (page.pgno >> 8) & 15;
+	if (!_vbi_export_grow_buffer_space (e, needed))
+		return FALSE;
 
-	h.hour = 0;
-	h.minute = 0;
+	h = (struct header *)(e->buffer.data + e->buffer.offset);
 
-	h.charset = page.national & 7;
+	memcpy (h->signature, "VTXV4", 5);
 
-	h.wst_flags = page.flags & C4_ERASE_PAGE;
-	h.wst_flags |= vbi_bit_reverse[page.flags >> 12];
-	h.vtx_flags = (0 << 7) | (0 << 6) | (0 << 5) | (0 << 4) | (0 << 3);
+	h->pagenum_l = page.pgno & 0xFF;
+	h->pagenum_h = (page.pgno >> 8) & 15;
+
+	h->hour = 0;
+	h->minute = 0;
+
+	h->charset = page.national & 7;
+
+	h->wst_flags = page.flags & C4_ERASE_PAGE;
+	h->wst_flags |= vbi_rev8 (page.flags >> 12);
+	h->vtx_flags = (0 << 7) | (0 << 6) | (0 << 5) | (0 << 4) | (0 << 3);
 	/* notfound, pblf (?), hamming error, virtual, seven bits */
 
-	if (fwrite(&h, sizeof(h), 1, fp) != 1)
-		goto write_error;
+	e->buffer.offset += sizeof (*h);
 
-	if (fwrite(page.data.lop.raw, 40 * 24, 1, fp) != 1)
-		goto write_error;
-
-	return TRUE;
-
-write_error:
-	vbi_export_write_error(e);
-	return FALSE;
+	return vbi_export_write (e, page.data.lop.raw, 40 * 24);
 }
 
 static vbi_export_info
@@ -146,3 +156,12 @@ vbi_export_class_vtx = {
 };
 
 VBI_AUTOREG_EXPORT_MODULE(vbi_export_class_vtx)
+
+#endif /* 0 */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
