@@ -1,23 +1,25 @@
 /*
- *  libzvbi - Teletext IDL packet demultiplexer
+ *  libzvbi -- Teletext Independent Data Line packet demultiplexer
  *
  *  Copyright (C) 2005 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: idl_demux.c,v 1.4 2006/05/26 00:46:33 mschimek Exp $ */
+/* $Id: idl_demux.c,v 1.11 2013/07/10 11:37:23 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -81,8 +83,10 @@ idl_a_demux_feed		(vbi_idl_demux *	dx,
 	unsigned int i;
 	unsigned int j;
 
-	if ((ial = vbi_unham8 (buffer[3])) < 0)
+	ial = vbi_unham8 (buffer[3]);
+	if (ial < 0) {
 		return FALSE;
+	}
 
 	spa_length = (unsigned int) ial & 7;
 	if (7 == spa_length) /* reserved */
@@ -93,8 +97,9 @@ idl_a_demux_feed		(vbi_idl_demux *	dx,
 	for (i = 0; i < spa_length; ++i)
 		spa |= vbi_unham8 (buffer[4 + i]) << (4 * i);
 
-	if (spa < 0)
+	if (spa < 0) {
 		return FALSE;
+	}
 
 	if (spa != dx->address)
 		return TRUE;
@@ -118,7 +123,7 @@ idl_a_demux_feed		(vbi_idl_demux *	dx,
 	}
 
 	if (0 != crc) {
-		if (!(ri & RI_PACKET_REPEATS)) {
+		if (0 == (ri & RI_PACKET_REPEATS)) {
 			/* Packet is corrupt and won't repeat. */
 
 			dx->ci = -1;
@@ -293,7 +298,7 @@ vbi_idl_demux_reset		(vbi_idl_demux *	dx)
  * function given to vbi_idl_a_demux_new() when new user data is available.
  *
  * @returns
- * FALSE if the packet contained incorrectable errors.
+ * FALSE if the packet contained uncorrectable errors.
  *
  * @since 0.2.14
  */
@@ -351,6 +356,39 @@ vbi_idl_demux_feed		(vbi_idl_demux *	dx,
 	}
 }
 
+/**
+ * @param dx IDL demultiplexer allocated with vbi_idl_a_demux_new().
+ * @param sliced Sliced VBI data.
+ * @param n_lines Number of lines in the @a sliced array.
+ *
+ * This function works like vbi_idl_demux_feed() but operates
+ * on sliced VBI data and filters out @c VBI_SLICED_TELETEXT_B_625.
+ *
+ * @returns
+ * FALSE if any Teletext lines contained uncorrectable errors.
+ *
+ * @since 0.2.26
+ */
+vbi_bool
+vbi_idl_demux_feed_frame	(vbi_idl_demux *	dx,
+				 const vbi_sliced *	sliced,
+				 unsigned int		n_lines)
+{
+	const vbi_sliced *end;
+
+	assert (NULL != dx);
+	assert (NULL != sliced);
+
+	for (end = sliced + n_lines; sliced < end; ++sliced) {
+		if (sliced->id & VBI_SLICED_TELETEXT_B_625) {
+			if (!vbi_idl_demux_feed (dx, sliced->data))
+				return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
 /** @internal */
 void
 _vbi_idl_demux_destroy		(vbi_idl_demux *	dx)
@@ -426,7 +464,7 @@ vbi_idl_demux_delete		(vbi_idl_demux *	dx)
 
 	_vbi_idl_demux_destroy (dx);
 
-	free (dx);		
+	vbi_free (dx);		
 }
 
 /**
@@ -454,15 +492,24 @@ vbi_idl_a_demux_new		(unsigned int		channel,
 {
 	vbi_idl_demux *dx;
 
+	assert (NULL != callback);
+
 	if (!(dx = vbi_malloc (sizeof (*dx)))) {
 		return NULL;
 	}
 
 	if (!_vbi_idl_demux_init (dx, _VBI_IDL_FORMAT_A, channel, address,
 				  callback, user_data)) {
-		free (dx);
+		vbi_free (dx);
 		dx = NULL;
 	}
 
 	return dx;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
