@@ -1,24 +1,26 @@
 /*
- *  libzvbi - FreeBSD/OpenBSD bktr driver interface
+ *  libzvbi -- FreeBSD/OpenBSD bktr driver interface
  *
  *  Copyright (C) 2002 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
 static const char rcsid [] =
-"$Id: io-bktr.c,v 1.13 2006/02/10 06:25:37 mschimek Exp $";
+"$Id: io-bktr.c,v 1.17 2008/02/19 00:35:20 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -199,7 +201,7 @@ vbi_capture_bktr_new		(const char *		dev_name,
 		"libzvbi interface rev.\n  %s\n", rcsid);
 
 	if (!(v = (vbi_capture_bktr *) calloc(1, sizeof(*v)))) {
-		vbi_asprintf(errstr, _("Virtual memory exhausted."));
+		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -212,8 +214,8 @@ vbi_capture_bktr_new		(const char *		dev_name,
 
 	v->fd = device_open (v->capture.sys_log_fp, dev_name, O_RDONLY, 0);
 	if (-1 == v->fd) {
-		vbi_asprintf(errstr, _("Cannot open '%s': %s."),
-			     dev_name, strerror(errno));
+		asprintf(errstr, _("Cannot open '%s': %s."),
+			 dev_name, strerror(errno));
 		goto io_error;
 	}
 
@@ -268,10 +270,10 @@ vbi_capture_bktr_new		(const char *		dev_name,
 		*services = vbi_raw_decoder_add_services (&v->dec, *services, strict);
 
 		if (*services == 0) {
-			vbi_asprintf(errstr, _("Sorry, %s (%s) cannot "
-						 "capture any of "
-						 "the requested data services."),
-				     dev_name, driver_name);
+			asprintf(errstr, _("Sorry, %s (%s) cannot "
+					   "capture any of "
+					   "the requested data services."),
+				 dev_name, driver_name);
 			goto failure;
 		}
 
@@ -280,7 +282,7 @@ vbi_capture_bktr_new		(const char *		dev_name,
 			       * sizeof(vbi_sliced));
 
 		if (!v->sliced_buffer.data) {
-			vbi_asprintf(errstr, _("Virtual memory exhausted."));
+			asprintf(errstr, _("Virtual memory exhausted."));
 			errno = ENOMEM;
 			goto failure;
 		}
@@ -298,7 +300,7 @@ vbi_capture_bktr_new		(const char *		dev_name,
 	v->raw_buffer = calloc(1, sizeof(v->raw_buffer[0]));
 
 	if (!v->raw_buffer) {
-		vbi_asprintf(errstr, _("Virtual memory exhausted."));
+		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -309,9 +311,9 @@ vbi_capture_bktr_new		(const char *		dev_name,
 	v->raw_buffer[0].data = malloc(v->raw_buffer[0].size);
 
 	if (!v->raw_buffer[0].data) {
-		vbi_asprintf(errstr, _("Not enough memory to allocate "
-					 "vbi capture buffer (%d KB)."),
-			     (v->raw_buffer[0].size + 1023) >> 10);
+		asprintf(errstr, _("Not enough memory to allocate "
+				   "vbi capture buffer (%d KB)."),
+			 (v->raw_buffer[0].size + 1023) >> 10);
 		goto failure;
 	}
 
@@ -385,10 +387,17 @@ vbi_capture_bktr_new		(const char *		dev_name,
 	printv ("Libzvbi bktr interface rev.\n  %s\n", rcsid);
 
 	if (errstr)
-		vbi_asprintf (errstr,
-			      _("BKTR driver interface not compiled."));
+		asprintf (errstr,
+			  _("BKTR driver interface not compiled."));
 
 	return NULL;
 }
 
 #endif /* !ENABLE_BKTR */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
