@@ -1,29 +1,31 @@
 /*
- *  libzvbi - VBI device simulation
+ *  libzvbi -- VBI device simulation
  *
- *  Copyright (C) 2004 Michael H. Schimek
+ *  Copyright (C) 2004, 2007 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: io-sim.c,v 1.11 2006/09/24 03:08:41 mschimek Exp $ */
+/* $Id: io-sim.c,v 1.18 2009/12/14 23:43:40 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <math.h>		/* sin() */
+#include <math.h>		/* sin(), log() */
 #include <errno.h>
 #include <ctype.h>		/* isspace() */
 #include <limits.h>		/* INT_MAX */
@@ -34,10 +36,22 @@
 #include "sampling_par.h"
 #include "raw_decoder.h"
 #include "hamm.h"
-#if 3 == VBI_VERSION_MINOR
+
+#if 2 == VBI_VERSION_MINOR
+#  define sp_sample_format sampling_format
+#  define SAMPLES_PER_LINE(sp)						\
+	((sp)->bytes_per_line / VBI_PIXFMT_BPP ((sp)->sampling_format))
+#  define SYSTEM_525(sp)						\
+	(525 == (sp)->scanning)
+#else
 #  include "vps.h"
 #  include "wss.h"
+#  define sp_sample_format sample_format
+#  define SAMPLES_PER_LINE(sp) ((sp)->samples_per_line)
+#  define SYSTEM_525(sp)						\
+	(0 != (VBI_VIDEOSTD_SET_525_60 & (sp)->videostd_set))
 #endif
+
 #include "io-sim.h"
 
 /**
@@ -67,7 +81,7 @@ do {									\
 	if (_vbi_global_log.mask & VBI_LOG_WARNING)			\
 		_vbi_log_printf (_vbi_global_log.fn,			\
 				  _vbi_global_log.user_data,		\
-				  VBI_LOG_WARNING, function,		\
+				  VBI_LOG_WARNING, __FILE__, function,	\
 				  templ , ##args);			\
 } while (0)
 
@@ -107,11 +121,23 @@ do {									\
 	PULSE (zero_level);						\
 } while (0)
 
-#if 3 == VBI_VERSION_MINOR
-#  define SAMPLES_PER_LINE(sp) ((sp)->samples_per_line)
-#else
-#  define SAMPLES_PER_LINE(sp)						\
-	((sp)->bytes_per_line / VBI_PIXFMT_BPP ((sp)->sampling_format))
+#ifndef HAVE_SINCOS
+
+/* This is a GNU extension. */
+_vbi_inline void
+sincos				(double			x,
+				 double *		sinx,
+				 double *		cosx)
+{
+	*sinx = sin (x);
+	*cosx = cos (x);
+}
+
+#endif
+
+/* This is a GNU extension. */
+#ifndef HAVE_LOG2
+#  define log2(x) (log (x) / M_LN2)
 #endif
 
 static void
@@ -302,9 +328,9 @@ signal_closed_caption		(uint8_t *		raw,
 	double t3 = t0 + 6.5 * D - 120e-9;
 	double q1 = PI * bit_rate * 2;
 	/* Max. rise/fall time 240 ns (EIA 608-B). */
-	double q2 = PI / 120e-9; 
-	double signal_mean = (white_level - blank_level) * .25; /* 25 IRE */
-	double signal_high = blank_level + (white_level - blank_level) * .5;
+	double q2 = PI / 120e-9;
+	double signal_mean;
+	double signal_high;
 	double sample_period = 1.0 / sp->sampling_rate;
 	unsigned int samples_per_line;
 	double t;
@@ -320,13 +346,22 @@ signal_closed_caption		(uint8_t *		raw,
 	samples_per_line = SAMPLES_PER_LINE (sp);
 
 	if (flags & _VBI_RAW_SHIFT_CC_CRI) {
-		/* Wrong signal shape found by Rich Kadel on
-		   "channel 56 The History Channel". */
+		/* Wrong signal shape found by Rich Kadel,
+		   zapping-misc@lists.sourceforge.net 2006-07-16. */
 		t0 += D / 2;
 		t1 += D / 2;
 		t2 += D / 2;
 	}
 
+	if (flags & _VBI_RAW_LOW_AMP_CC) {
+		/* Low amplitude signal found by Rich Kadel,
+		   zapping-misc@lists.sourceforge.net 2007-08-15. */
+		white_level = white_level * 6 / 10;
+	}
+
+	signal_mean = (white_level - blank_level) * .25; /* 25 IRE */
+	signal_high = blank_level + (white_level - blank_level) * .5;
+
 	for (i = 0; i < samples_per_line; ++i) {
 		if (t >= t1 && t < t2) {
 			raw[i] = SATURATE (blank_level
@@ -383,6 +418,149 @@ clear_image			(uint8_t *		p,
 	}
 }
 
+/**
+ * @param raw Noise will be added to this raw VBI image.
+ * @param sp Describes the raw VBI data in the buffer. @a sp->sampling_format
+ *   must be @c VBI_PIXFMT_Y8 (@c VBI_PIXFMT_YUV420 in libzvbi 0.2.x).
+ *   Note for compatibility in libzvbi 0.2.x vbi_sampling_par is a
+ *   synonym of vbi_raw_decoder, but the (private) decoder fields in
+ *   this structure are ignored.
+ * @param min_freq Minimum frequency of the noise in Hz.
+ * @param max_freq Maximum frequency of the noise in Hz. @a min_freq and
+ *   @a max_freq define the cut off frequency at the half power points
+ *   (gain -3 dB).
+ * @param amplitude Maximum amplitude of the noise, should lie in range
+ *   0 to 256.
+ * @param seed Seed for the pseudo random number generator built into
+ *   this function. Given the same @a seed value the function will add
+ *   the same noise, which can be useful for tests.
+ *
+ * This function adds white noise to a raw VBI image.
+ *
+ * To produce realistic noise @a min_freq = 0, @a max_freq = 5e6 and
+ * @a amplitude = 20 to 50 seems appropriate.
+ *
+ * @returns
+ * FALSE if the @a sp sampling parameters are invalid.
+ *
+ * @since 0.2.26
+ */
+vbi_bool
+vbi_raw_add_noise		(uint8_t *		raw,
+				 const vbi_sampling_par *sp,
+				 unsigned int		min_freq,
+				 unsigned int		max_freq,
+				 unsigned int		amplitude,
+				 unsigned int		seed)
+{
+	double f0, w0, sn, cs, bw, alpha, a0;
+	float a1, a2, b0, b1, z0, z1, z2;
+	unsigned int n_lines;
+	unsigned long samples_per_line;
+	unsigned long padding;
+	uint32_t seed32;
+
+	assert (NULL != raw);
+	assert (NULL != sp);
+
+	if (unlikely (!_vbi_sampling_par_valid_log (sp, /* log */ NULL)))
+		return FALSE;
+
+	switch (sp->sp_sample_format) {
+#if 3 == VBI_VERSION_MINOR
+	case VBI_PIXFMT_YUV444:
+	case VBI_PIXFMT_YVU444:
+	case VBI_PIXFMT_YUV422:
+	case VBI_PIXFMT_YVU422:
+	case VBI_PIXFMT_YUV411:
+	case VBI_PIXFMT_YVU411:
+	case VBI_PIXFMT_YVU420:
+	case VBI_PIXFMT_YUV410:
+	case VBI_PIXFMT_YVU410:
+	case VBI_PIXFMT_Y8:
+#endif
+	case VBI_PIXFMT_YUV420:
+		break;
+
+	default:
+		return FALSE;
+	}
+
+	if (unlikely (sp->sampling_rate <= 0))
+		return FALSE;
+
+	/* Biquad bandpass filter.
+	   http://www.musicdsp.org/files/Audio-EQ-Cookbook.txt */
+
+	f0 = ((double) min_freq + max_freq) * 0.5;
+
+	if (f0 <= 0.0)
+		return TRUE;
+
+	w0 = 2 * M_PI * f0 / sp->sampling_rate;
+	sincos (w0, &sn, &cs);
+	bw = fabs (log2 (MAX (min_freq, max_freq) / f0));
+	alpha = sn * sinh (log (2) / 2 * bw * w0 / sn);
+	a0 = 1 + alpha;
+	a1 = 2 * cs / a0;
+	a2 = (alpha - 1) / a0;
+	b0 = sn / (2*a0);
+	b1 = 0;
+
+	if (amplitude > 256)
+		amplitude = 256;
+
+	n_lines = sp->count[0] + sp->count[1];
+
+#if 2 == VBI_VERSION_MINOR
+	if (unlikely (0 == amplitude
+		      || 0 == n_lines
+		      || 0 == sp->bytes_per_line))
+		return TRUE;
+
+	samples_per_line = sp->bytes_per_line;
+	padding = 0;
+#else
+	if (unlikely (0 == amplitude
+		      || 0 == n_lines
+		      || 0 == sp->samples_per_line))
+		return TRUE;
+
+	samples_per_line = sp->samples_per_line;
+	padding = sp->bytes_per_line - samples_per_line;
+#endif
+
+	seed32 = seed;
+
+	z1 = 0;
+	z2 = 0;
+
+	do {
+		uint8_t *raw_end = raw + samples_per_line;
+
+		do {
+			int noise;
+
+			/* We use our own simple PRNG to produce
+			   predictable results for tests. */
+			seed32 = seed32 * 1103515245u + 12345;
+			noise = ((seed32 / 65536) % (amplitude * 2 + 1))
+				- amplitude;
+
+			z0 = noise + a1 * z1 + a2 * z2;
+			noise = (int)(b0 * (z0 - z2) + b1 * z1);
+			z2 = z1;
+			z1 = z0;
+
+			*raw++ = SATURATE (*raw + noise, 0, 255);
+		} while (raw < raw_end);
+
+		raw += padding;
+	} while (--n_lines > 0);
+
+	return TRUE;
+}
+
 static vbi_bool
 signal_u8			(uint8_t *		raw,
 				 const vbi_sampling_par *sp,
@@ -568,7 +746,8 @@ _vbi_raw_vbi_image		(uint8_t *		raw,
 	n_scan_lines = sp->count[0] + sp->count[1];
 	if (unlikely (n_scan_lines * sp->bytes_per_line > raw_size)) {
 		warning (__FUNCTION__,
-			 "%u + %u lines * %lu bytes_per_line > %lu raw_size.",
+			 "(%u + %u lines) * %lu bytes_per_line "
+			 "> %lu raw_size.",
 			 sp->count[0], sp->count[1],
 			 (unsigned long) sp->bytes_per_line, raw_size);
 		return FALSE;
@@ -581,11 +760,7 @@ _vbi_raw_vbi_image		(uint8_t *		raw,
 			 blank_level, white_level);
 	}
 
-#if 3 == VBI_VERSION_MINOR
-	if (VBI_VIDEOSTD_SET_525_60 & sp->videostd_set) {
-#else
-	if (525 == sp->scanning) {
-#endif
+	if (SYSTEM_525 (sp)) {
 		/* Observed value. */
 		const unsigned int peak = 200; /* 255 */
 
@@ -743,7 +918,7 @@ _vbi_raw_video_image		(uint8_t *		raw,
 			 blank_level, black_level, white_level);
 	}
 
-	switch (sp->sampling_format) {
+	switch (sp->sp_sample_format) {
 #if 3 == VBI_VERSION_MINOR
 	case VBI_PIXFMT_YVUA24_LE:	/* 0xAAUUVVYY */
 	case VBI_PIXFMT_YVU24_LE:	/* 0x00UUVVYY */
@@ -803,7 +978,7 @@ _vbi_raw_video_image		(uint8_t *		raw,
 		break;
 	}
 
-	switch (sp->sampling_format) {
+	switch (sp->sp_sample_format) {
 	case VBI_PIXFMT_RGB16_LE:
 	case VBI_PIXFMT_RGB16_BE:
 	case VBI_PIXFMT_BGR16_LE:
@@ -869,11 +1044,7 @@ _vbi_raw_video_image		(uint8_t *		raw,
 
 	/* ITU-R BT.601 sampling assumed. */
 
-#if 3 == VBI_VERSION_MINOR
-	if (VBI_VIDEOSTD_SET_525_60 & sp->videostd_set) {
-#else
-	if (525 == sp->scanning) {
-#endif
+	if (SYSTEM_525 (sp)) {
 		if (0 == white_level) {
 			/* Cutting off the bottom of the signal
 			   confuses the vbi_bit_slicer (can't adjust
@@ -897,7 +1068,7 @@ _vbi_raw_video_image		(uint8_t *		raw,
 	samples_per_line = SAMPLES_PER_LINE (sp);
 
 #if 3 == VBI_VERSION_MINOR
-	sp8.sampling_format = VBI_PIXFMT_Y8;
+	sp8.sample_format = VBI_PIXFMT_Y8;
 #else
 	sp8.sampling_format = VBI_PIXFMT_YUV420;
 #endif
@@ -927,7 +1098,7 @@ _vbi_raw_video_image		(uint8_t *		raw,
 	while (n_scan_lines-- > 0) {
 		unsigned int i;
 
-		switch (sp->sampling_format) {
+		switch (sp->sp_sample_format) {
 #if 3 == VBI_VERSION_MINOR
 		case VBI_PIXFMT_NONE:
 		case VBI_PIXFMT_RESERVED0:
@@ -946,6 +1117,9 @@ _vbi_raw_video_image		(uint8_t *		raw,
 		case VBI_PIXFMT_YUV410:
 		case VBI_PIXFMT_YVU410:
 		case VBI_PIXFMT_Y8:
+#endif
+#if 2 == VBI_VERSION_MINOR
+		case VBI_PIXFMT_PAL8:
 #endif
 		case VBI_PIXFMT_YUV420:
 			for (i = 0; i < samples_per_line; ++i)
@@ -1268,8 +1442,84 @@ typedef struct {
 	uint8_t			vps_buffer[13];
 
 	uint8_t			wss_buffer[2];
+
+	unsigned int		noise_min_freq;
+	unsigned int		noise_max_freq;
+	unsigned int		noise_amplitude;
+	unsigned int		noise_seed;
+
+	unsigned int		flags;
 } vbi_capture_sim;
 
+unsigned int
+_vbi_capture_sim_get_flags	(vbi_capture *		cap)
+{
+	vbi_capture_sim *sim;
+
+	assert (NULL != cap);
+
+	sim = PARENT (cap, vbi_capture_sim, cap);
+	assert (MAGIC == sim->magic);
+
+	return sim->flags;
+}
+
+void
+_vbi_capture_sim_set_flags	(vbi_capture *		cap,
+				 unsigned int		flags)
+{
+	vbi_capture_sim *sim;
+
+	assert (NULL != cap);
+
+	sim = PARENT (cap, vbi_capture_sim, cap);
+	assert (MAGIC == sim->magic);
+
+	sim->flags = flags;
+}
+
+/**
+ * @param cap Initialized vbi_capture context opened with
+ *   vbi_capture_sim_new().
+ * @param min_freq Minimum frequency of the noise in Hz.
+ * @param max_freq Maximum frequency of the noise in Hz. @a min_freq and
+ *   @a max_freq define the cut off frequency at the half power points
+ *   (gain -3 dB).
+ * @param amplitude Maximum amplitude of the noise, should lie in range
+ *   0 to 256.
+ *
+ * This function shapes the white noise to be added to simulated raw VBI
+ * data. By default no noise is added. To disable the noise set
+ * @a amplitude to zero.
+ *
+ * To produce realistic noise @a min_freq = 0, @a max_freq = 5e6 and
+ * @a amplitude = 20 to 50 seems appropriate.
+ *
+ * @since 0.2.26
+ */
+void
+vbi_capture_sim_add_noise	(vbi_capture *		cap,
+				 unsigned int		min_freq,
+				 unsigned int		max_freq,
+				 unsigned int		amplitude)
+{
+	vbi_capture_sim *sim;
+
+	assert (NULL != cap);
+
+	sim = PARENT (cap, vbi_capture_sim, cap);
+	assert (MAGIC == sim->magic);
+
+	if (0 == max_freq)
+		amplitude = 0;
+
+	sim->noise_min_freq = min_freq;
+	sim->noise_max_freq = max_freq;
+	sim->noise_amplitude = amplitude;
+
+	sim->noise_seed = 123456789;
+}
+
 static vbi_bool
 extend_buffer			(struct buffer *	b,
 				 unsigned int		new_capacity)
@@ -1288,13 +1538,13 @@ extend_buffer			(struct buffer *	b,
 
 static const char
 caption_default_test_stream [] =
-	"<erase-displayed ch=\"0\"/><roll-up rows=\"4\"/><pac row=\"14\"/>"
+	"<edm ch=\"0\"/><ru4/><pac row=\"15\"/>"
 	"LIBZVBI CAPTION SIMULATION CC1.<cr/>"
-	"<erase-displayed ch=\"1\"/><roll-up rows=\"4\"/><pac row=\"14\"/>"
+	"<edm ch=\"1\"/><ru4/><pac row=\"15\"/>"
 	"LIBZVBI CAPTION SIMULATION CC2.<cr/>"
-	"<erase-displayed ch=\"2\"/><roll-up rows=\"4\"/><pac row=\"14\"/>"
+	"<edm ch=\"2\"/><ru4/><pac row=\"15\"/>"
 	"LIBZVBI CAPTION SIMULATION CC3.<cr/>"
-	"<erase-displayed ch=\"3\"/><roll-up rows=\"4\"/><pac row=\"14\"/>"
+	"<edm ch=\"3\"/><ru4/><pac row=\"15\"/>"
 	"LIBZVBI CAPTION SIMULATION CC4.<cr/>"
 ;
 	/* TODO: regression test for repeated control code bug:
@@ -1302,17 +1552,20 @@ caption_default_test_stream [] =
 	   <zero field 2>
 	   <repeated control code field 1, to be ignored> */ 
 
-static unsigned int
-get_attr			(const char *		s,
+static vbi_bool
+get_attr			(unsigned int *		value,
+				 const char *		s,
 				 const char *		name,
 				 unsigned int		default_value,
 				 unsigned int		minimum,
 				 unsigned int		maximum)
 {
-	unsigned long value;
+	unsigned long u;
 	unsigned int len;
+	vbi_bool present;
 
-	value = default_value;
+	present = FALSE;
+	u = default_value;
 
 	len = strlen (name);
 
@@ -1337,9 +1590,8 @@ get_attr			(const char *		s,
 			break;
 
 		if (0 == delta) {
-			value = strtoul (s + 1,
-					 /* endp */ NULL,
-					 /* base */ 0);
+			present = TRUE;
+			u = strtoul (s + 1, NULL, 0);
 			break;
 		}
 
@@ -1347,21 +1599,20 @@ get_attr			(const char *		s,
 		while (0 != *s && '"' != *s);
 	}
 
-	value = SATURATE (value,
-			  (unsigned long) minimum,
-			  (unsigned long) maximum);
+	*value = SATURATE (u, (unsigned long) minimum,
+			   (unsigned long) maximum);
 
-	return (unsigned int) value;
+	return present;
 }
 
 static vbi_bool
 caption_append_zeroes		(vbi_capture_sim *	sim,
-				 unsigned int		channel,
+				 vbi_pgno		channel,
 				 unsigned int		n_bytes)
 {
 	struct buffer *b;
 
-	b = &sim->caption_buffers[(channel >> 1) & 1];
+	b = &sim->caption_buffers[((channel - 1) >> 1) & 1];
 
 	if (b->size + n_bytes > b->capacity) {
 		unsigned int new_capacity;
@@ -1385,8 +1636,6 @@ caption_append_command		(vbi_capture_sim *	sim,
 	static const _vbi_key_value_pair elements [] = {
 		{ "aof", 0x1422 },
 		{ "aon", 0x1423 },
-		{ "backgr", 0x1020 },
-		{ "backgr-transp", 0x172D },
 		{ "bao", 0x102E },
 		{ "bas", 0x102F },
 		{ "bbo", 0x1024 },
@@ -1407,39 +1656,26 @@ caption_append_command		(vbi_capture_sim *	sim,
 		{ "bys", 0x102B },
 		{ "cmd", 0x0001 },
 		{ "cr", 0x142D },
-		{ "delete-end-of-row", 0x1424 },
 		{ "der", 0x1424 },
 		{ "edm", 0x142C },
-		{ "end-of-caption", 0x142F },
 		{ "enm", 0x142E },
 		{ "eoc", 0x142F },
-		{ "erase-displayed", 0x142C },
-		{ "erase-non-displayed", 0x142E },
-		{ "extended2", 0x1200 },
-		{ "extended3", 0x1300 },
+		{ "ext2", 0x1200 },
+		{ "ext3", 0x1300 },
 		{ "fa", 0x172E },
 		{ "fau", 0x172F },
-		{ "flash-on", 0x1428 },
 		{ "fon", 0x1428 },
-		{ "foregr-black", 0x172E },
-		{ "indent", 0x1050 },
 		{ "mr", 0x1120 },
 		{ "pac", 0x1040 },
 		{ "pause", 0x0002 },
 		{ "rcl", 0x1420 },
 		{ "rdc", 0x1429 },
-		{ "resume-caption", 0x1420 },
-		{ "resume-direct", 0x1429 },
-		{ "resume-text", 0x142B },
-		{ "roll-up", 0x1425 },
 		{ "rtd", 0x142B },
 		{ "ru2", 0x1425 },
 		{ "ru3", 0x1426 },
 		{ "ru4", 0x1427 },
-		{ "special", 0x1130 },
+		{ "spec", 0x1130 },
 		{ "sync", 0x0003 },
-		{ "tab", 0x1720 },
-		{ "text-restart", 0x142A },
 		{ "to1", 0x1721 },
 		{ "to2", 0x1722 },
 		{ "to3", 0x1723 },
@@ -1455,114 +1691,105 @@ caption_append_command		(vbi_capture_sim *	sim,
 	int value;
 	unsigned int cmd;
 	unsigned int n_frames;
+	unsigned int u_value;
 	int n_padding_bytes;
-	unsigned int row;
 	unsigned int i;
 	vbi_bool parity;
 
 	if (!_vbi_keyword_lookup (&value, &s,
-				   elements,
-				   N_ELEMENTS (elements)))
+				  elements,
+				  N_ELEMENTS (elements)))
 		return TRUE;
 
-	*inout_ch = get_attr (s, "ch", *inout_ch, 0, 3);
+	get_attr (inout_ch, s, "ch", *inout_ch, 1, 4);
 
-	cmd = value | ((*inout_ch & 1) << 11);
+	cmd = value | (((*inout_ch - 1) & 1) << 11);
 
 	parity = TRUE;
 
 	switch (value) {
 	case 1: /* cmd */
-		cmd = get_attr (s, "code", 0, 0, 0xFFFF);
+		get_attr (&cmd, s, "code", 0, 0, 0xFFFF);
 		parity = FALSE;
 		break;
 
 	case 2: /* pause */
-		n_frames = get_attr (s, "frames", 60, 1, INT_MAX);
+		get_attr (&n_frames, s, "frames", 60, 1, INT_MAX);
 		if (n_frames > 120 * 60 * 30)
 			return TRUE;
 
-		return caption_append_zeroes (sim, *inout_ch, n_frames * 2);
+		return caption_append_zeroes
+			(sim, *inout_ch, n_frames * 2);
 
 	case 3: /* sync */
 		n_padding_bytes = sim->caption_buffers[0].size
 			- sim->caption_buffers[1].size;
 
-		if (0 == n_padding_bytes)
+		if (0 == n_padding_bytes) {
 			return TRUE;
-		else if (n_padding_bytes < 0)
-			return caption_append_zeroes (sim, 0, n_padding_bytes);
-		else
-			return caption_append_zeroes (sim, 2, n_padding_bytes);
+		} else if (n_padding_bytes < 0) {
+			return caption_append_zeroes
+				(sim, 0, n_padding_bytes);
+		} else {
+			return caption_append_zeroes
+				(sim, 2, n_padding_bytes);
+		}
 
-	case 0x1020: /* backgr */
-		cmd |= get_attr (s, "color", 0, 0, 7) << 1;
-		cmd |= get_attr (s, "t", 0, 0, 1); /* transparent */
+	case 0x1040: /* preamble address code */
+		if (get_attr (&u_value, s, "column", 1, 1, 32)) {
+			cmd |= 0x0010 | (((u_value - 1) / 4) << 1);
+		} else {
+			get_attr (&u_value, s, "color", 0, 0, 7);
+			cmd |= u_value << 1;
+		}
+		get_attr (&u_value, s, "row", 15, 1, 15);
+		cmd |= row_code[u_value - 1];
+		get_attr (&u_value, s, "u", 0, 0, 1);
+		cmd |= u_value;
 		break;
 
-	case 0x1040: /* pac (preamble address code) */
-		cmd |= row_code[get_attr (s, "row", 14, 0, 14)];
-		cmd |= get_attr (s, "color", 0, 0, 7) << 1;
-		cmd |= get_attr (s, "u", 0, 0, 1);
-		break;
-
-	case 0x1050: /* indent */
-		cmd |= row_code[get_attr (s, "row", 14, 0, 14)];
-		cmd |= (get_attr (s, "cols", 0, 0, 31) / 4) << 1;
-		cmd |= get_attr (s, "u", 0, 0, 1);
-		break;
-
-	case 0x1120: /* mr (midrow code) */
-		cmd |= get_attr (s, "color", 0, 0, 7) << 1;
-		cmd |= get_attr (s, "u", 0, 0, 1);
+	case 0x1120: /* midrow code */
+		get_attr (&u_value, s, "color", 0, 0, 7);
+		cmd |= u_value << 1;
+		get_attr (&u_value, s, "u", 0, 0, 1);
+		cmd |= u_value;
 		break;
 
 	case 0x1130: /* special character */
-		cmd |= get_attr (s, "code", 0, 0, 15);
+		get_attr (&u_value, s, "code", 0, 0, 15);
+		cmd |= u_value;
 		break;
 
 	case 0x1200: /* extended character set */
 	case 0x1300:
-		cmd |= get_attr (s, "code", 32, 32, 63);
+		get_attr (&u_value, s, "code", 32, 32, 63);
+		cmd |= u_value;
 		break;
 
-	case 0x1420: /* resume caption loading */
+	case 0x1420: /* rcl */
 	case 0x1421: /* bs */
 	case 0x1422: /* aof */
 	case 0x1423: /* aon */
-	case 0x1424: /* delete to end of row */
-	case 0x1428: /* flash-on */
-	case 0x1429: /* resume direct caption */
-	case 0x142A: /* text restart */
-	case 0x142B: /* resume text display */
-	case 0x142C: /* erase displayed memory */
+	case 0x1424: /* der */
+	case 0x1425: /* ru3 */
+	case 0x1426: /* ru4 */
+	case 0x1427: /* ru5 */
+	case 0x1428: /* fon */
+	case 0x1429: /* rdc */
+	case 0x142A: /* tr */
+	case 0x142B: /* rtd */
+	case 0x142C: /* edm */
 	case 0x142D: /* cr */
-	case 0x142E: /* erase non-displayed memory */
-	case 0x142F: /* end of caption */
+	case 0x142E: /* enm */
+	case 0x142F: /* eoc */
 		/* Field bit (EIA 608-B Sec. 8.4, 8.5). */
-		cmd |= ((*inout_ch & 2) << 7);
-
-	case 0x1425: /* roll_up */
-	case 0x1426:
-	case 0x1427:
-		row = get_attr (s, "rows", 2, 2, 4);
-		cmd += row - 2;
-		cmd |= (*inout_ch & 2) << 7; /* field bit */
-		break;
-
-	case 0x1720: /* tab */
-		cmd |= get_attr (s, "cols", 1, 1, 3);
-		break;
-
-	case 0x172E: /* foregr-black */
-		cmd |= get_attr (s, "u", 0, 0, 1); /* underlined */
-		break;
+		cmd |= ((*inout_ch - 1) & 2) << 7;
 
 	default:
 		break;
 	}
 
-	b = &sim->caption_buffers[(*inout_ch >> 1) & 1];
+	b = &sim->caption_buffers[((*inout_ch - 1) >> 1) & 1];
 	i = b->size;
 
 	if (i + 3 > b->capacity) {
@@ -1587,9 +1814,6 @@ caption_append_command		(vbi_capture_sim *	sim,
 	return TRUE;
 }
 
-#if 3 != VBI_VERSION_MINOR
-static
-#endif
 vbi_bool
 vbi_capture_sim_load_caption	(vbi_capture *		cap,
 				 const char *		stream,
@@ -1606,8 +1830,8 @@ vbi_capture_sim_load_caption	(vbi_capture *		cap,
 	assert (MAGIC == sim->magic);
 
 	if (!append) {
-		free (sim->caption_buffers[0].data);
-		free (sim->caption_buffers[1].data);
+		vbi_free (sim->caption_buffers[0].data);
+		vbi_free (sim->caption_buffers[1].data);
 
 		CLEAR (sim->caption_buffers);
 
@@ -1617,7 +1841,7 @@ vbi_capture_sim_load_caption	(vbi_capture *		cap,
 	if (NULL == stream)
 		return TRUE;
 
-	ch = 0;
+	ch = 1; /* CC1, T1 */
 
 	b = &sim->caption_buffers[0];
 
@@ -1645,6 +1869,12 @@ vbi_capture_sim_load_caption	(vbi_capture *		cap,
 			} else if (0 == strncmp (s, "gt;", 3)) {
 				s += 3;
 				c = '>';
+			} else if (0 == strncmp (s, "ts;", 3)) {
+				/* Transparent space. */
+				if (!caption_append_command
+				    (sim, &ch, "<spec code=\"9\"/>"))
+					return FALSE;
+				continue;
 			}
 		} else if ('<' == c) {
 			int delimiter;
@@ -1652,7 +1882,7 @@ vbi_capture_sim_load_caption	(vbi_capture *		cap,
 			if (!caption_append_command (sim, &ch, s))
 				return FALSE;
 
-			b = &sim->caption_buffers[(ch >> 1) & 1];
+			b = &sim->caption_buffers[((ch - 1) >> 1) & 1];
 
 			/* Skip until '>', except between quotes. */
 			delimiter = '>';
@@ -2092,11 +2322,7 @@ sim_read			(vbi_capture *		cap,
 
 	if (NULL != raw
 	    || NULL != sliced) {
-#if 3 == VBI_VERSION_MINOR
-		if (VBI_VIDEOSTD_SET_525_60 & sim->sp.videostd_set) {
-#else
-		if (525 == sim->sp.scanning) {
-#endif
+		if (SYSTEM_525 (&sim->sp)) {
 			n_lines = gen_sliced_525 (sim);
 		} else {
 			n_lines = gen_sliced_625 (sim);
@@ -2122,16 +2348,29 @@ sim_read			(vbi_capture *		cap,
 
 		memset (raw_data, 0x80, sim->raw_buffer.size);
 
-		success = vbi_raw_vbi_image (raw_data,
-					     sim->raw_buffer.size,
-					     &sim->sp,
-					     /* blank_level */ 0,
-					     /* white_level */ 0,
-					      /* swap_fields */ FALSE,
-					     sim->sliced,
-					     n_lines);
+		success = _vbi_raw_vbi_image (raw_data,
+					       sim->raw_buffer.size,
+					       &sim->sp,
+					       /* blank_level: default */ 0,
+					       /* white_level: default */ 0,
+					       sim->flags,
+					       sim->sliced,
+					       n_lines);
 		assert (success);
 
+		if (sim->noise_amplitude > 0) {
+			success = vbi_raw_add_noise (raw_data,
+						      &sim->sp,
+						      sim->noise_min_freq,
+						      sim->noise_max_freq,
+						      sim->noise_amplitude,
+						      sim->noise_seed);
+			assert (success);
+
+			sim->noise_seed = sim->noise_seed
+				* 1103515245 + 56789;
+		}
+
 		if (!sim->sp.synchronous)
 			delay_raw_data (sim, raw_data);
 
@@ -2163,11 +2402,7 @@ sim_read			(vbi_capture *		cap,
 		(*sliced)->timestamp = sim->capture_time;
 	}
 
-#if 3 == VBI_VERSION_MINOR
-	if (VBI_VIDEOSTD_SET_525_60 & sim->sp.videostd_set) {
-#else
-	if (525 == sim->sp.scanning) {
-#endif
+	if (SYSTEM_525 (&sim->sp)) {
 		sim->capture_time += 1001 / 30000.0;
 	} else {
 		sim->capture_time += 1 / 25.0;
@@ -2176,25 +2411,48 @@ sim_read			(vbi_capture *		cap,
 	return TRUE;
 }
 
+static vbi_bool
+sim_sampling_point		(vbi_capture *		cap,
+				 vbi3_bit_slicer_point *point,
+				 unsigned int		row,
+				 unsigned int		nth_bit)
+{
+	vbi_capture_sim *sim = PARENT (cap, vbi_capture_sim, cap);
+
+	if (!sim->decode_raw)
+		return FALSE;
+
+	return vbi3_raw_decoder_sampling_point (sim->rd, point, row, nth_bit);
+}
+
+static vbi_bool
+sim_debug			(vbi_capture *		cap,
+				 vbi_bool		enable)
+{
+	vbi_capture_sim *sim = PARENT (cap, vbi_capture_sim, cap);
+
+	return vbi3_raw_decoder_debug (sim->rd, enable);
+}
+
+/* For compatibility in libzvbi 0.2
+   struct vbi_sampling_par == vbi_raw_decoder. In 0.3
+   we'll drop the decoding related fields. */
+#if 3 == VBI_VERSION_MINOR
+static const vbi_sampling_par *
+#else
 static vbi_raw_decoder *
+#endif
 sim_parameters			(vbi_capture *		cap)
 {
 	vbi_capture_sim *sim = PARENT (cap, vbi_capture_sim, cap);
 
-	/* For compatibility in libzvbi 0.2
-	   struct vbi_sampling_par == vbi_raw_decoder. In 0.3
-	   we'll drop the decoding related fields. */
-#if 3 == VBI_VERSION_MINOR
-	return &sim->rd;
-#else
 	return &sim->sp;
-#endif
 }
 
 static int
 sim_get_fd			(vbi_capture *		cap)
 {
-	cap = cap;
+	cap = cap; /* unused */
 
 	return -1; /* not available */
 }
@@ -2210,14 +2468,14 @@ sim_delete			(vbi_capture *		cap)
 
 	vbi3_raw_decoder_delete (sim->rd);
 
-	free (sim->desync_buffer[1]);
-	free (sim->desync_buffer[0]);
+	vbi_free (sim->desync_buffer[1]);
+	vbi_free (sim->desync_buffer[0]);
 
-	free (sim->raw_buffer.data);
+	vbi_free (sim->raw_buffer.data);
 
 	CLEAR (*sim);
 
-	free (sim);
+	vbi_free (sim);
 }
 
 /**
@@ -2272,6 +2530,8 @@ vbi_capture_sim_new		(int			scanning,
 
 	sim->cap.read		= sim_read;
 	sim->cap.parameters	= sim_parameters;
+	sim->cap.debug		= sim_debug;
+	sim->cap.sampling_point	= sim_sampling_point;
 	sim->cap.get_fd		= sim_get_fd;
 	sim->cap._delete	= sim_delete;
 
@@ -2298,7 +2558,7 @@ vbi_capture_sim_new		(int			scanning,
 	sim->raw_f2_size = sim->sp.bytes_per_line * sim->sp.count[1];
 
 	sim->raw_buffer.size = sim->raw_f1_size + sim->raw_f2_size;
-	sim->raw_buffer.data = malloc (sim->raw_buffer.size);
+	sim->raw_buffer.data = vbi_malloc (sim->raw_buffer.size);
 	if (NULL == sim->raw_buffer.data) {
 		goto failure;
 	}
@@ -2325,7 +2585,7 @@ vbi_capture_sim_new		(int			scanning,
 	/* Raw VBI decoder. */
 
 	sim->rd = vbi3_raw_decoder_new (&sim->sp);
-	if (0 == sim->rd) {
+	if (NULL == sim->rd) {
 		goto failure;
 	}
 
@@ -2365,3 +2625,10 @@ vbi_capture_sim_new		(int			scanning,
 
 	return NULL;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
