@@ -1,25 +1,27 @@
 /*
- *  libzvbi - V4L interface
+ *  libzvbi -- Video For Linux driver interface
  *
  *  Copyright (C) 1999-2004 Michael H. Schimek
  *  Copyright (C) 2003, 2004 Tom Zoerner
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
 static const char rcsid [] =
-"$Id: io-v4l.c,v 1.32 2006/04/29 05:55:35 mschimek Exp $";
+"$Id: io-v4l.c,v 1.39 2013/07/02 04:04:04 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -40,6 +42,7 @@ static const char rcsid [] =
 #include <assert.h>
 #include <sys/time.h>		/* timeval */
 #include <sys/types.h>		/* fd_set, uid_t */
+#include <sys/stat.h>		/* S_ISCHR */
 #include <sys/ioctl.h>		/* for (_)videodev.h */
 #include <pthread.h>
 
@@ -62,7 +65,7 @@ static const char rcsid [] =
 /* Custom ioctl of the bttv driver. */
 #define BTTV_VBISIZE		_IOR('v' , BASE_VIDIOCPRIVATE+8, int)
 static __inline__ void IOCTL_ARG_TYPE_CHECK_BTTV_VBISIZE
-  (const int *arg __attribute__ ((unused))) {}
+  (const int *arg _vbi_unused) {}
 
 #undef REQUIRE_SELECT
 #undef REQUIRE_SVBIFMT		/* else accept current parameters */
@@ -159,7 +162,7 @@ v4l_read_alloc(vbi_capture_v4l *v, char ** errstr)
 	v->raw_buffer = calloc(1, sizeof(v->raw_buffer[0]));
 
 	if (v->raw_buffer == NULL) {
-		vbi_asprintf(errstr, _("Virtual memory exhausted."));
+		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -170,9 +173,9 @@ v4l_read_alloc(vbi_capture_v4l *v, char ** errstr)
 	v->raw_buffer[0].data = malloc(v->raw_buffer[0].size);
 
 	if (v->raw_buffer[0].data == NULL) {
-		vbi_asprintf(errstr, _("Not enough memory to allocate "
-					 "vbi capture buffer (%d KB)."),
-			     (v->raw_buffer[0].size + 1023) >> 10);
+		asprintf(errstr, _("Not enough memory to allocate "
+				   "vbi capture buffer (%d KB)."),
+			 (v->raw_buffer[0].size + 1023) >> 10);
 		goto failure;
 	}
 
@@ -753,7 +756,7 @@ set_parameters(vbi_capture_v4l *v, struct vbi_format *p_vfmt, int *p_max_rate,
 					          dec_temp.scanning, p_max_rate);
 
 	if ((sup_services & *services) == 0) {
-		vbi_asprintf(errstr, _("Sorry, %s (%s) cannot capture any of the "
+		asprintf(errstr, _("Sorry, %s (%s) cannot capture any of the "
 					 "requested data services."),
 			     v->p_dev_name, v->vcap.name);
 		return FALSE;
@@ -804,9 +807,9 @@ set_parameters(vbi_capture_v4l *v, struct vbi_format *p_vfmt, int *p_max_rate,
 		*p_vfmt = vfmt_temp;
 		return TRUE;
 #endif
-		vbi_asprintf(errstr, _("Cannot initialize %s (%s), "
-					 "the device is already in use."),
-			     v->p_dev_name, v->vcap.name);
+		asprintf(errstr, _("Cannot initialize %s (%s), "
+				   "the device is already in use."),
+			 v->p_dev_name, v->vcap.name);
 		break;
 
 	case EINVAL:
@@ -818,8 +821,8 @@ set_parameters(vbi_capture_v4l *v, struct vbi_format *p_vfmt, int *p_max_rate,
                 }
 		break;
 	default:
-		vbi_asprintf(errstr, _("Could not set the vbi "
-					 "capture parameters for %s (%s): %s."),
+		asprintf(errstr, _("Could not set the vbi "
+				   "capture parameters for %s (%s): %s."),
 			     v->p_dev_name, v->vcap.name, strerror(errno));
 		/* guess = _("Maybe a bug in the driver or libzvbi."); */
 		break;
@@ -956,11 +959,12 @@ v4l_update_services(vbi_capture *vc,
 		printv("Accept current vbi parameters\n");
 
 		if (vfmt.sample_format != VIDEO_PALETTE_RAW) {
-			vbi_asprintf(errstr, _("%s (%s) offers unknown vbi "
-						 "sampling format #%d. "
-						 "This may be a driver bug "
-						 "or libzvbi is too old."),
-				     v->p_dev_name, v->vcap.name, vfmt.sample_format);
+			asprintf(errstr, _("%s (%s) offers unknown vbi "
+					   "sampling format #%d. "
+					   "This may be a driver bug "
+					   "or libzvbi is too old."),
+				 v->p_dev_name, v->vcap.name,
+				 vfmt.sample_format);
 			goto io_error;
 		}
 
@@ -1008,9 +1012,9 @@ v4l_update_services(vbi_capture *vc,
 
 		if (0 && !strstr(v->vcap.name, "bttv")
 		      && !strstr(v->vcap.name, "BTTV")) {
-			vbi_asprintf(errstr, _("Cannot capture with %s (%s), "
-						 "has no standard vbi interface."),
-				     v->p_dev_name, v->vcap.name);
+			asprintf(errstr, _("Cannot capture with %s (%s), "
+					   "has no standard vbi interface."),
+				 v->p_dev_name, v->vcap.name);
 			goto io_error;
 		}
 
@@ -1027,11 +1031,11 @@ v4l_update_services(vbi_capture *vc,
 			v->dec.count[0] = 16;
 			v->dec.count[1] = 16;
 		} else if (size % 2048) {
-			vbi_asprintf (errstr,
-				      _("Cannot identify %s (%s), reported "
-					"vbi frame size suggests this is "
-					"not a bttv driver."),
-				      v->p_dev_name, v->vcap.name);
+			asprintf (errstr,
+				  _("Cannot identify %s (%s), reported "
+				    "vbi frame size suggests this is "
+				    "not a bttv driver."),
+				  v->p_dev_name, v->vcap.name);
 			goto io_error;
 		} else {
 			printv ("Driver supports BTTV_VBISIZE: %d bytes, "
@@ -1045,9 +1049,9 @@ v4l_update_services(vbi_capture *vc,
 		switch (v->dec.scanning) {
 		default:
 #ifdef REQUIRE_VIDEOSTD
-			vbi_asprintf(errstr, _("Cannot set or determine current "
-						 "videostandard of %s (%s)."),
-				     v->p_dev_name, v->vcap.name);
+			asprintf(errstr, _("Cannot set or determine current "
+					   "videostandard of %s (%s)."),
+				 v->p_dev_name, v->vcap.name);
 			goto io_error;
 #endif
 			printv("Warning: Videostandard not confirmed, "
@@ -1084,12 +1088,12 @@ v4l_update_services(vbi_capture *vc,
 		/* Nyquist */
 
 		if (v->dec.sampling_rate < max_rate * 3 / 2) {
-			vbi_asprintf(errstr, _("Cannot capture the requested "
-						 "data services with "
-						 "%s (%s), the sampling frequency "
-						 "%.2f MHz is too low."),
-				     v->p_dev_name, v->vcap.name,
-				     v->dec.sampling_rate / 1e6);
+			asprintf(errstr, _("Cannot capture the requested "
+					   "data services with "
+					   "%s (%s), the sampling frequency "
+					   "%.2f MHz is too low."),
+				 v->p_dev_name, v->vcap.name,
+				 v->dec.sampling_rate / 1e6);
                         services = 0;
 			goto failure;
 		}
@@ -1112,10 +1116,10 @@ v4l_update_services(vbi_capture *vc,
 							         strict);
 
 		if (services == 0) {
-			vbi_asprintf(errstr, _("Sorry, %s (%s) cannot "
-						 "capture any of "
-						 "the requested data services."),
-				     v->p_dev_name, v->vcap.name);
+			asprintf(errstr, _("Sorry, %s (%s) cannot "
+					   "capture any of "
+					   "the requested data services."),
+				 v->p_dev_name, v->vcap.name);
 			goto failure;
 		}
 
@@ -1127,7 +1131,7 @@ v4l_update_services(vbi_capture *vc,
 			       * sizeof(vbi_sliced));
 
 		if (!v->sliced_buffer.data) {
-			vbi_asprintf(errstr, _("Virtual memory exhausted."));
+			asprintf(errstr, _("Virtual memory exhausted."));
 			errno = ENOMEM;
 			goto failure;
 		}
@@ -1165,7 +1169,7 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 		scanning = 0;
 
 	if (!(v = (vbi_capture_v4l *) calloc(1, sizeof(*v)))) {
-		vbi_asprintf(errstr, _("Virtual memory exhausted."));
+		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -1193,7 +1197,7 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 	v->p_dev_name = strdup(dev_name);
 
 	if (v->p_dev_name == NULL) {
-		vbi_asprintf(errstr, _("Virtual memory exhausted."));
+		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -1201,8 +1205,8 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 	v->fd = device_open (v->capture.sys_log_fp,
 			     v->p_dev_name, O_RDONLY, 0);
 	if (-1 == v->fd) {
-		vbi_asprintf (errstr, _("Cannot open '%s': %d, %s."),
-			      v->p_dev_name, errno, strerror(errno));
+		asprintf (errstr, _("Cannot open '%s': %d, %s."),
+			  v->p_dev_name, errno, strerror(errno));
 		perm_check(v, v->p_dev_name);
 		goto io_error;
 	}
@@ -1218,7 +1222,7 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 		 *  v4l ioctls, let's see if we can guess the beast.
 		 */
 		printv("Driver doesn't support VIDIOCGCAP\n");
-		strncpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
+		strlcpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
 		v->vcap.name[sizeof(v->vcap.name) - 1] = 0;
 
 		if (!guess_bttv_v4l(v, &strict, given_fd, scanning))
@@ -1227,14 +1231,14 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 		if (v->vcap.name[0] != 0) {
 			printv("Driver name '%s'\n", v->vcap.name);
 		} else {
-			strncpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
+			strlcpy(v->vcap.name, _("driver unknown"), sizeof(v->vcap.name) - 1);
 			v->vcap.name[sizeof(v->vcap.name) - 1] = 0;
 		}
 
 		if (!(v->vcap.type & VID_TYPE_TELETEXT)) {
-			vbi_asprintf(errstr,
-				     _("%s (%s) is not a raw vbi device."),
-				     v->p_dev_name, v->vcap.name);
+			asprintf(errstr,
+				 _("%s (%s) is not a raw vbi device."),
+				 v->p_dev_name, v->vcap.name);
 			goto failure;
 		}
 
@@ -1253,9 +1257,9 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
 
 #ifdef REQUIRE_SELECT
 	if (!v->select) {
-		vbi_asprintf(errstr, _("%s (%s) does not support "
-					 "the select() function."),
-			     v->p_dev_name, v->vcap.name);
+		asprintf(errstr, _("%s (%s) does not support "
+				   "the select() function."),
+			 v->p_dev_name, v->vcap.name);
 		goto failure;
 	}
 #endif
@@ -1283,9 +1287,9 @@ v4l_new(const char *dev_name, int given_fd, int scanning,
                                  *  ourselves, but then we had guessed already.
                                  */
 #ifdef REQUIRE_VIDEOSTD
-                                vbi_asprintf(errstr, _("Cannot set or determine current "
-                                                         "videostandard of %s (%s)."),
-                                             v->p_dev_name, v->vcap.name);
+                                asprintf(errstr, _("Cannot set or determine current "
+						   "videostandard of %s (%s)."),
+					 v->p_dev_name, v->vcap.name);
                                 goto failure;
 #endif
                                 printv("Warning: Videostandard not confirmed, "
@@ -1387,7 +1391,7 @@ vbi_capture_v4l_sidecar_new(const char *dev_name, int given_fd,
 	pthread_once (&vbi_init_once, vbi_init);
 
 	if (errstr)
-		vbi_asprintf(errstr, _("V4L driver interface not compiled."));
+		asprintf(errstr, _("V4L driver interface not compiled."));
 
 	return NULL;
 }
@@ -1432,9 +1436,16 @@ vbi_capture_v4l_new(const char *dev_name, int scanning,
 		fprintf (stderr, "Libzvbi V4L interface rev.\n  %s\n", rcsid);
 
 	if (errstr)
-		vbi_asprintf (errstr, _("V4L driver interface not compiled."));
+		asprintf (errstr, _("V4L driver interface not compiled."));
 
 	return NULL;
 }
 
 #endif /* !ENABLE_V4L */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
