@@ -1,24 +1,26 @@
 /*
- *  libzvbi - V4L2 interface
+ *  libzvbi -- Video For Linux Two 0.20 driver interface
  *
  *  Copyright (C) 1999-2004 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
 static const char rcsid [] =
-"$Id: io-v4l2.c,v 1.32 2006/02/10 06:25:37 mschimek Exp $";
+"$Id: io-v4l2.c,v 1.37 2008/02/19 00:35:20 mschimek Exp $";
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -41,6 +43,14 @@ static const char rcsid [] =
 #include <asm/types.h>		/* for videodev2.h */
 #include <pthread.h>
 
+#ifndef HAVE_S64_U64
+#  include <inttypes.h>
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+
 #include "videodev2.h"
 #include "_videodev2.h"
 
@@ -102,7 +112,7 @@ vbi_capture_v4l2_new		(const char *		dev_name,
 	*errstr = NULL;
 
 	if (!(v = calloc (1, sizeof (*v)))) {
-		vbi_asprintf (errstr, _("Virtual memory exhausted."));
+		asprintf (errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
@@ -117,8 +127,8 @@ vbi_capture_v4l2_new		(const char *		dev_name,
 		v->fd = device_open (v->capture.sys_log_fp,
 				     dev_name, O_RDONLY, 0);
 		if (-1 == v->fd) {
-			vbi_asprintf (errstr, _("Cannot open '%s': %d, %s."),
-				      dev_name, errno, strerror (errno));
+			asprintf (errstr, _("Cannot open '%s': %d, %s."),
+				  dev_name, errno, strerror (errno));
 			goto io_error;
 		}
 	}
@@ -127,8 +137,8 @@ vbi_capture_v4l2_new		(const char *		dev_name,
 
 	if (-1 == xioctl (v, VIDIOC_QUERYCAP, &v->vcap)) {
 		/* TRANSLATORS: Cannot identify '/dev/some'. */
-		/* vbi_asprintf (errstr, _("Cannot identify '%s': %s."),
-		   		 dev_name, strerror (errno)); */
+		/* asprintf (errstr, _("Cannot identify '%s': %s."),
+		             dev_name, strerror (errno)); */
 
 		v4l2_delete (&v->capture);
 
@@ -145,7 +155,7 @@ vbi_capture_v4l2_new		(const char *		dev_name,
 	}
 
 	/* XXX localize. */
-	vbi_asprintf (errstr, "V4L2 0.20 API not supported.");
+	asprintf (errstr, "V4L2 0.20 API not supported.");
 
  io_error:
  failure:
@@ -208,10 +218,17 @@ vbi_capture_v4l2_new		(const char *		dev_name,
 		fprintf (stderr, "Libzvbi V4L2 interface rev.\n  %s\n", rcsid);
 
 	if (errstr)
-		vbi_asprintf (errstr,
-			      _("V4L2 driver interface not compiled."));
+		asprintf (errstr,
+			  _("V4L2 driver interface not compiled."));
 
 	return NULL;
 }
 
 #endif /* !ENABLE_V4L2 */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
