@@ -1,25 +1,28 @@
 /*
- *  libzvbi - V4L2 (version 2002-10 and later) interface
+ *  libzvbi -- Video For Linux Two (version 2002-10 and later)
+ *             driver interface
  *
  *  Copyright (C) 2002-2005 Michael H. Schimek
- *  Copyright (C) 2003-2004 Tom Zoerner
+ *  Copyright (C) 2003, 2004 Tom Zoerner
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
 static const char rcsid [] =
-"$Id: io-v4l2k.c,v 1.42 2006/09/24 03:10:04 mschimek Exp $";
+"$Id: io-v4l2k.c,v 1.50 2009/12/14 23:43:20 mschimek Exp $";
 
 /*
  *  Around Oct-Nov 2002 the V4L2 API was revised for inclusion into
@@ -49,12 +52,20 @@ static const char rcsid [] =
 #include <sys/types.h>		/* fd_set */
 #include <sys/ioctl.h>		/* for (_)videodev2k.h */
 #include <sys/mman.h>		/* PROT_READ, MAP_SHARED */
-#include <asm/types.h>		/* for videodev2k.h */
+#include <asm/types.h>		/* __u8 and friends for videodev2k.h */
 #include <pthread.h>
 
 #include "raw_decoder.h"
 #include "version.h"
 
+#ifndef HAVE_S64_U64
+#  include <inttypes.h>
+  /* Linux 2.6.x asm/types.h defines __s64 and __u64 only
+     if __GNUC__ is defined. */
+typedef int64_t __s64;
+typedef uint64_t __u64;
+#endif
+
 #include "videodev2k.h"
 #include "_videodev2k.h"
 
@@ -230,6 +241,7 @@ v4l2_stream_alloc(vbi_capture_v4l2 *v, char ** errstr)
 
 		vbuf.type = v->btype;
 		vbuf.index = v->num_raw_buffers;
+		vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QUERYBUF, &vbuf)) {
 			asprintf (errstr,
@@ -336,9 +348,10 @@ restart_stream			(vbi_capture_v4l2 *	v)
 
 		vbuf.index = i;
 		vbuf.type = v->btype;
+		vbuf.memory = V4L2_MEMORY_MMAP;
 
-		if (-1 == xioctl (v, VIDIOC_QBUF, &vbuf))
-			return FALSE;
+		/* Error ignored. */
+		xioctl (v, VIDIOC_QBUF, &vbuf);
 	}
 
 	if (-1 == xioctl (v, VIDIOC_STREAMON, &v->btype))
@@ -373,6 +386,7 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 	} else if (ENQUEUE_IS_UNQUEUED(v->enqueue)) {
 		v->vbuf.type = v->btype;
 		v->vbuf.index = v->enqueue;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QBUF, &v->vbuf)) {
 			error (&v->log,
@@ -399,6 +413,7 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 		}
 
 		v->vbuf.type = v->btype;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		/* retrieve the captured frame from the queue */
 		r = xioctl (v, VIDIOC_DQBUF, &v->vbuf);
@@ -411,10 +426,9 @@ v4l2_stream(vbi_capture *vc, vbi_capture_buffer **raw,
 			       "Failed to dequeue buffer, errno %d.",
 			       errno);
 
-			/* On EIO bttv dequeues the buffer, or it does not,
-			   or it resets the hardware (SCERR).  QBUF alone
-			   is insufficient.
-			   Actually the caller should restart on error. */
+			/* On EIO bttv dequeues the buffer, other drivers
+			   may not. Actually the caller should restart
+			   on error. */
 
 			/* Errors ignored. */
 			restart_stream (v);
@@ -502,6 +516,7 @@ v4l2_stream_flush(vbi_capture *vc)
 	if (ENQUEUE_IS_UNQUEUED(v->enqueue)) {
 		v->vbuf.type = v->btype;
 		v->vbuf.index = v->enqueue;
+		v->vbuf.memory = V4L2_MEMORY_MMAP;
 
 		if (-1 == xioctl (v, VIDIOC_QBUF, &v->vbuf)) {
 			error (&v->log,
@@ -815,8 +830,11 @@ print_vfmt			(vbi_capture_v4l2 *	v,
 	if (0 == (v->log.mask & VBI_LOG_INFO))
 		return;
 
-	_vbi_log_printf (v->log.fn, v->log.user_data,
-			 VBI_LOG_INFO, __FUNCTION__,
+	_vbi_log_printf (v->log.fn,
+			 v->log.user_data,
+			 VBI_LOG_INFO,
+			 __FILE__,
+			 __FUNCTION__,
 			 "%sformat %08x [%c%c%c%c], %d Hz, %d bpl, offs %d, "
 			 "F1 %d...%d, F2 %d...%d, flags %08x.", s,
 			 vfmt->fmt.vbi.sample_format,
@@ -1058,11 +1076,12 @@ v4l2_update_services(vbi_capture *vc,
 		    && 9 == vfmt.fmt.vbi.start[0]
 		    && 272 == vfmt.fmt.vbi.start[1]) {
 			/* Captures only 288 * 4 samples/line,
-			   work-around not possible. */
+			   no work-around possible. */
+			/* XXX when was this fixed? */
 			asprintf (errstr,
 				  _("A known bug in driver %s %u.%u.%u "
 				    "impedes VBI capturing in NTSC mode. "
-				    "Please upgrade."),
+				    "Please upgrade the driver."),
 				  v->vcap.driver,
 				  (v->vcap.version >> 16) & 0xFF,
 				  (v->vcap.version >> 8) & 0xFF,
@@ -1297,6 +1316,7 @@ vbi_capture_v4l2k_new		(const char *		dev_name,
 	char *guess = NULL;
 	char *error = NULL;
 	vbi_capture_v4l2 *v;
+	_vbi_log_hook log;
 
 	pthread_once (&vbi_init_once, vbi_init);
 
@@ -1309,18 +1329,23 @@ vbi_capture_v4l2k_new		(const char *		dev_name,
 		errstr = &error;
 	*errstr = NULL;
 
+	CLEAR (log);
+	if (trace) {
+		log.fn = vbi_log_on_stderr;
+		log.mask = VBI_LOG_INFO * 2 - 1;
+	}
+
 	if (!(v = calloc(1, sizeof(*v)))) {
 		asprintf(errstr, _("Virtual memory exhausted."));
 		errno = ENOMEM;
 		goto failure;
 	}
 
+	v->log = log;
+
 	_vbi3_raw_decoder_init (&v->rd, /* sampling_par */ NULL);
 
 	if (trace) {
-		v->log.fn = vbi_log_on_stderr;
-		v->log.mask = VBI_LOG_INFO * 2 - 1;
-
 		vbi3_raw_decoder_set_log_fn (&v->rd,
 					     vbi_log_on_stderr,
 					     /* user_data */ NULL,
@@ -1463,13 +1488,13 @@ vbi_capture_v4l2k_new		(const char *		dev_name,
 
  io_error:
  failure:
-	if (v)
-		v4l2_delete (&v->capture);
-
-	info (&v->log,
+	info ((NULL != v) ? &v->log : &log,
 	      "Failed with errno %d, errmsg '%s'.",
 	      errno, *errstr);
 
+	if (NULL != v)
+		v4l2_delete (&v->capture);
+
 	if (errstr == &error) {
 		free (error);
 		error = NULL;
@@ -1530,3 +1555,10 @@ vbi_capture_v4l2k_new(const char *dev_name, int fd, int buffers,
 }
 
 #endif /* !ENABLE_V4L2 */
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
