@@ -1330,7 +1330,7 @@ vbi_bool vbi_proxy_msg_finish_connect( int sock_fd, char ** ppErrorText )
 ** Query size and character of an ioctl request for v4l1 drivers
 */
 static int
-vbi_proxy_msg_v4l_ioctl( int request, void * p_arg, vbi_bool * req_perm )
+vbi_proxy_msg_v4l_ioctl( unsigned int request, void * p_arg, vbi_bool * req_perm )
 {
    p_arg = p_arg;
 
@@ -1374,7 +1374,7 @@ vbi_proxy_msg_v4l_ioctl( int request, void * p_arg, vbi_bool * req_perm )
 ** Query size and character of an ioctl request for v4l2 drivers
 */
 static int
-vbi_proxy_msg_v4l2_ioctl( int request, void * p_arg, vbi_bool * req_perm )
+vbi_proxy_msg_v4l2_ioctl( unsigned int request, void * p_arg, vbi_bool * req_perm )
 {
    switch (request)
    {
