@@ -1,23 +1,25 @@
 /*
- *  libzvbi - Raw VBI decoder
+ *  libzvbi -- Raw VBI decoder
  *
  *  Copyright (C) 2000-2004 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: raw_decoder.c,v 1.8 2006/04/29 05:55:35 mschimek Exp $ */
+/* $Id: raw_decoder.c,v 1.24 2008/08/19 10:04:46 mschimek Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -26,39 +28,21 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
+
 #include "misc.h"
 #include "raw_decoder.h"
 
-#define vbi_inline static __inline__
-
 #ifndef RAW_DECODER_PATTERN_DUMP
-#  define DECODER_PATTERN_DUMP 0
+#  define RAW_DECODER_PATTERN_DUMP 0
 #endif
 
-#ifndef RAW_DECODER_LOG
-#  define RAW_DECODER_LOG 0
+#if 2 == VBI_VERSION_MINOR
+#  define sp_sample_format sampling_format
+#else
+#  define sp_sample_format sample_format
 #endif
 
-#ifndef PRIx64
-#  define PRIx64 "llx"
-#endif
-#ifndef PRId64
-#  define PRId64 "lld"
-#endif
-
-#define sp_log(level, templ, args...)					\
-do {									\
-	vbi_log_printf (log_fn, log_user_data,				\
-			level, __FUNCTION__, templ , ##args);		\
-} while (0)
-
-#define rd_log(level, templ, args...)					\
-do {									\
-	if (level <= rd->log_max_level)					\
-		vbi_log_printf (rd->log_fn, rd->log_user_data,		\
-				level, __FUNCTION__, templ , ##args);	\
-} while (0)
-
 /**
  * $addtogroup RawDecoder Raw VBI decoder
  * $ingroup Raw
@@ -69,6 +53,7 @@ do {									\
    VITC PAL 6-22 11.2us 1.8125 Mbit NRZ two start bits + CRC
    VITC NTSC 10-21 ditto
    CGMS NTSC 20 11us .450450 Mbit NRZ ?
+   MOJI
 */
 const _vbi_service_par
 _vbi_service_table [] = {
@@ -124,7 +109,8 @@ _vbi_service_table [] = {
 		{ 16, 0 },
 		{ 16, 0 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8,
+		VBI_MODULATION_BIPHASE_MSB,
 		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_VPS_F2, "Pseudo-VPS on field 2",
@@ -132,7 +118,8 @@ _vbi_service_table [] = {
 		{ 0, 329 },
 		{ 0, 329 },
 		12500, 5000000, 2500000, /* 160 x FH */
-		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8, VBI_MODULATION_BIPHASE_MSB,
+		0xAAAA8A99, 0xFFFFFF, 32, 0, 13 * 8,
+		VBI_MODULATION_BIPHASE_MSB,
 		_VBI_SP_FIELD_NUM,
 	}, {
 		VBI_SLICED_WSS_625, "Wide Screen Signalling 625",
@@ -140,9 +127,11 @@ _vbi_service_table [] = {
 		{ 23, 0 },
 		{ 23, 0 },
 		11000, 5000000, 833333, /* 160/3 x FH */
-		0xC71E3C1F, 0x924C99CE, 32, 0, 14 * 1,
+		/* ...1000 111 / 0 0011 1100 0111 1000 0011 111x */
+		/* ...0010 010 / 0 1001 1001 0011 0011 1001 110x */	
+		0x8E3C783E, 0x2499339C, 32, 0, 14 * 1,
 		VBI_MODULATION_BIPHASE_LSB,
-		/* Hm. Easily confused with caption?? */
+		/* Hm. Too easily confused with caption?? */
 		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_CAPTION_625_F1, "Closed Caption 625, field 1",
@@ -214,7 +203,11 @@ _vbi_service_table [] = {
 		{ 21, 0 },
 		{ 21, 0 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		/* Test of CRI bits has been removed to handle the
+		   incorrect signal observed by Rich Kandel (see
+		   _VBI_RAW_SHIFT_CC_CRI). */
+		0x03, 0x0F, 4, 0, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		/* 0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB, */
 		/* I've seen CC signals on other lines and there's no
 		   way to distinguish from the transmitted data. */
 		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
@@ -225,7 +218,8 @@ _vbi_service_table [] = {
 		{ 0, 284 },
 		{ 0, 284 },
 		10500, 1006976, 503488, /* 32 x FH */
-		0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		0x03, 0x0F, 4, 0, 2 * 8, VBI_MODULATION_NRZ_LSB,
+		/* 0x00005551, 0x7FF, 14, 2, 2 * 8, VBI_MODULATION_NRZ_LSB, */
 		_VBI_SP_FIELD_NUM | _VBI_SP_LINE_NUM,
 	}, {
 		VBI_SLICED_2xCAPTION_525, /* NOT CONFIRMED */
@@ -256,7 +250,7 @@ _vbi_service_table [] = {
 	}
 };
 
-vbi_inline const _vbi_service_par *
+_vbi_inline const _vbi_service_par *
 find_service_par		(unsigned int		service)
 {
 	unsigned int i;
@@ -288,6 +282,8 @@ vbi_sliced_name			(vbi_service_set	service)
 		return "Closed Caption 625";
 	if (service == (VBI_SLICED_VPS | VBI_SLICED_VPS_F2))
 		return "Video Program System";
+	if (service == VBI_SLICED_TELETEXT_B_L25_625)
+		return "Teletext System B 625 Level 2.5";
 
 	/* Incorrect, no longer in table */
 	if (service == VBI_SLICED_TELETEXT_BD_525)
@@ -300,12 +296,12 @@ vbi_sliced_name			(vbi_service_set	service)
 }
 
 /**
- * $ingroup Sliced
- * $param service A data service identifier, for example from a
+ * @ingroup Sliced
+ * @param service A data service identifier, for example from a
  *   vbi_sliced structure.
  *
- * $return
- * Number of payload bits, $c 0 if the service is unknown.
+ * @return
+ * Number of payload bits, @c 0 if the service is unknown.
  */
 unsigned int
 vbi_sliced_payload_bits		(unsigned int		service)
@@ -319,6 +315,8 @@ vbi_sliced_payload_bits		(unsigned int		service)
 		return 16;
 	if (service == (VBI_SLICED_VPS | VBI_SLICED_VPS_F2))
 		return 13 * 8;
+	if (service == VBI_SLICED_TELETEXT_B_L25_625)
+		return 42 * 8;
 
 	/* Incorrect, no longer in table */
 	if (service == VBI_SLICED_TELETEXT_BD_525)
@@ -409,7 +407,7 @@ _vbi3_raw_decoder_dump		(const vbi3_raw_decoder *rd,
 	}
 }
 
-vbi_inline int
+_vbi_inline int
 cpr1204_crc			(const vbi_sliced *	sliced)
 {
 	const int poly = (1 << 6) + (1 << 1) + 1;
@@ -429,7 +427,32 @@ cpr1204_crc			(const vbi_sliced *	sliced)
 	return crc;
 }
 
-vbi_inline vbi_sliced *
+static vbi_bool
+slice				(vbi3_raw_decoder *	rd,
+				 vbi_sliced *		sliced,
+				 _vbi3_raw_decoder_job *job,
+				 unsigned int		i,
+				 const uint8_t *	raw)
+{
+	if (rd->debug && NULL != rd->sp_lines) {
+		return vbi3_bit_slicer_slice_with_points
+			(&job->slicer,
+			 sliced->data,
+			 sizeof (sliced->data),
+			 rd->sp_lines[i].points,
+			 &rd->sp_lines[i].n_points,
+			 N_ELEMENTS (rd->sp_lines[i].points),
+			 raw);
+	} else {
+		return vbi3_bit_slicer_slice
+			(&job->slicer,
+			 sliced->data,
+			 sizeof (sliced->data),
+			 raw);
+	}
+}
+
+_vbi_inline vbi_sliced *
 decode_pattern			(vbi3_raw_decoder *	rd,
 				 vbi_sliced *		sliced,
 				 int8_t *		pattern,
@@ -451,8 +474,7 @@ decode_pattern			(vbi3_raw_decoder *	rd,
 
 			job = rd->jobs + j - 1;
 
-			if (!job->slicer.func (&job->slicer,
-					       sliced->data, raw)) {
+			if (!slice (rd, sliced, job, i, raw)) {
 				continue; /* no match, try next data service */
 			}
 
@@ -477,6 +499,8 @@ decode_pattern			(vbi3_raw_decoder *	rd,
 
 			/* Positive match, output decoded line. */
 
+			/* FIXME: if we have a field number we should
+			   really only set the service id of one field. */
 			sliced->id = job->id;
 			sliced->line = 0;
 
@@ -546,7 +570,7 @@ decode_pattern			(vbi3_raw_decoder *	rd,
  *   vbi scan line may contain data, this should be an array of vbi_sliced
  *   with the same number of elements as scan lines in the raw image
  *   (vbi_sampling_parameters.count[0] + .count[1]).
- * $param sliced_lines Size of $a sliced data array, in lines, not bytes.
+ * $param max_lines Size of $a sliced data array, in lines, not bytes.
  * $param raw A raw vbi image as described by the vbi_sampling_par
  *   associated with $a rd.
  * 
@@ -564,7 +588,7 @@ decode_pattern			(vbi3_raw_decoder *	rd,
 unsigned int
 vbi3_raw_decoder_decode		(vbi3_raw_decoder *	rd,
 				 vbi_sliced *		sliced,
-				 unsigned int		sliced_lines,
+				 unsigned int		max_lines,
 				 const uint8_t *	raw)
 {
 	vbi_sampling_par *sp;
@@ -589,9 +613,9 @@ vbi3_raw_decoder_decode		(vbi3_raw_decoder *	rd,
 	raw1 = raw;
 
 	sliced_begin = sliced;
-	sliced_end = sliced + sliced_lines;
+	sliced_end = sliced + max_lines;
 
-	if (DECODER_PATTERN_DUMP)
+	if (RAW_DECODER_PATTERN_DUMP)
 		_vbi3_raw_decoder_dump (rd, stderr);
 
 	for (i = 0; i < scan_lines; ++i) {
@@ -612,442 +636,6 @@ vbi3_raw_decoder_decode		(vbi3_raw_decoder *	rd,
 	return sliced - sliced_begin;
 }
 
-vbi_bool
-_vbi_sampling_par_valid		(const vbi_sampling_par *sp,
-				 vbi_log_fn *		log_fn,
-				 void *			log_user_data)
-{
-	switch (sp->sampling_format) {
-	case VBI_PIXFMT_YUV420:
-		/* This conflicts with the ivtv driver, which returns an
-		   odd number of bytes per line.  The driver format is
-		   _GREY but libzvbi 0.2 has no VBI_PIXFMT_Y8. */
-		/* if (sp->bytes_per_line & 1)
-		   goto samples; */
-		break;
-
-	default:
-		if (0 != (sp->bytes_per_line
-			  % VBI_PIXFMT_BPP (sp->sampling_format)))
-			goto samples;
-		break;
-	}
-
-	if (0 == sp->count[0]
-	    && 0 == sp->count[1])
-		goto range;
-
-	if (525 == sp->scanning) {
-		if (0 != sp->start[0])
-			if ((sp->start[0] + sp->count[0]) > 265)
-				goto range;
-
-		if (0 != sp->start[1])
-			if (sp->start[1] < 263
-			    || (sp->start[1] + sp->count[0]) > 526)
-				goto range;
-	} else if (625 == sp->scanning) {
-		if (0 != sp->start[0])
-			if ((sp->start[0] + sp->count[0]) > 336)
-				goto range;
-
-		if (0 != sp->start[1])
-			if (sp->start[1] < 310
-			    || (sp->start[1] + sp->count[0]) > 626)
-				goto range;
-	} else {
-		sp_log (VBI_LOG_ERR,
-			"Ambiguous scanning %d.",
-			sp->scanning);
-		return FALSE;
-	}
-
-	if (sp->interlaced
-	    && (sp->count[0] != sp->count[1]
-		|| 0 == sp->count[0])) {
-		sp_log (VBI_LOG_ERR,
-			"Line counts %u, %u must be equal and "
-			"non-zero when raw VBI data is interlaced.",
-			sp->count[0], sp->count[1]);
-		return FALSE;
-	}
-
-	return TRUE;
-
- samples:
-	sp_log (VBI_LOG_ERR,
-		"bytes_per_line value %u is no multiple of "
-		"the sample size %u.",
-		sp->bytes_per_line,
-		VBI_PIXFMT_BPP (sp->sampling_format));
-	return FALSE;
-
- range:
-	sp_log (VBI_LOG_ERR,
-		"Invalid VBI scan range %u-%u (%u lines), "
-		"%u-%u (%u lines).",
-		sp->start[0], sp->start[0] + sp->count[0] - 1,
-		sp->count[0],
-		sp->start[1], sp->start[1] + sp->count[1] - 1,
-		sp->count[1]);
-
-	return FALSE;
-}
-
-/* Attn: strict must be int for compatibility with libzvbi 0.2 (-1 == 0) */
-static vbi_bool
-_vbi_sampling_par_check_service	(const vbi_sampling_par *sp,
-				 const _vbi_service_par *par,
-				 int			strict,
-				 vbi_log_fn *		log_fn,
-				 void *			log_user_data)
-{
-	const unsigned int unknown = 0;
-	double signal;
-	unsigned int field;
-	unsigned int samples_per_line;
-	vbi_videostd_set videostd_set;
-
-	switch (sp->scanning) {
-	case 525:
-		videostd_set = VBI_VIDEOSTD_SET_525_60;
-		break;
-
-	case 625:
-		videostd_set = VBI_VIDEOSTD_SET_625_50;
-		break;
-
-	default:
-		videostd_set = 0;
-		break;
-	}
-
-	if (0 == (par->videostd_set & videostd_set)) {
-		vbi_log_printf (log_fn, log_user_data,
-				VBI_LOG_NOTICE, __FUNCTION__,
-				"Service 0x%08x (%s) requires "
-				"videostd_set 0x%" PRIx64 ", "
-				"have 0x%" PRIx64,
-				par->id, par->label,
-				par->videostd_set, videostd_set);
-		return FALSE;
-	}
-
-	if (par->flags & _VBI_SP_LINE_NUM) {
-		if ((par->first[0] > 0 && unknown == sp->start[0])
-		    || (par->first[1] > 0 && unknown == sp->start[1])) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Service 0x%08x (%s) requires known "
-					"line numbers.",
-					par->id, par->label);
-			return FALSE;
-		}
-	}
-
-	{
-		unsigned int rate;
-
-		rate = MAX (par->cri_rate, par->bit_rate);
-
-		switch (par->id) {
-		case VBI_SLICED_WSS_625:
-			/* Effective bit rate is just 1/3 max_rate,
-			   so 1 * max_rate should suffice. */
-			break;
-
-		default:
-			rate = (rate * 3) >> 1;
-			break;
-		}
-
-		if (rate > (unsigned int) sp->sampling_rate) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Sampling rate %f MHz too low "
-					"for service 0x%08x (%s).",
-					sp->sampling_rate / 1e6,
-					par->id, par->label);
-			return FALSE;
-		}
-	}
-
-	signal = par->cri_bits / (double) par->cri_rate
-		+ (par->frc_bits + par->payload) / (double) par->bit_rate;
-
-	samples_per_line = sp->bytes_per_line
-		/ VBI_PIXFMT_BPP (sp->sampling_format);
-
-	if (sp->offset > 0 && strict > 0) {
-		double sampling_rate;
-		double offset;
-		double end;
-
-		sampling_rate = (double) sp->sampling_rate;
-
-		offset = sp->offset / sampling_rate;
-		end = (sp->offset + samples_per_line) / sampling_rate;
-
-		if (offset > (par->offset / 1e3 - 0.5e-6)) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Sampling starts at 0H + %f us, too "
-					"late for service 0x%08x (%s) at "
-					"%f us.",
-					offset * 1e6,
-					par->id, par->label,
-					par->offset / 1e3);
-			return FALSE;
-		}
-
-		if (end < (par->offset / 1e9 + signal + 0.5e-6)) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Sampling ends too early at 0H + "
-					"%f us for service 0x%08x (%s) "
-					"which ends at %f us",
-					end * 1e6,
-					par->id, par->label,
-					par->offset / 1e3
-					+ signal * 1e6 + 0.5);
-			return FALSE;
-		}
-	} else {
-		double samples;
-
-		samples = samples_per_line / (double) sp->sampling_rate;
-
-		if (samples < (signal + 1.0e-6)) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Service 0x%08x (%s) signal length "
-					"%f us exceeds %f us sampling length.",
-					par->id, par->label,
-					signal * 1e6, samples * 1e6);
-			return FALSE;
-		}
-	}
-
-	if ((par->flags & _VBI_SP_FIELD_NUM)
-	    && !sp->synchronous) {
-		vbi_log_printf (log_fn, log_user_data,
-				VBI_LOG_NOTICE, __FUNCTION__,
-				"Service 0x%08x (%s) requires "
-				"synchronous field order.",
-				par->id, par->label);
-		return FALSE;
-	}
-
-	for (field = 0; field < 2; ++field) {
-		unsigned int start;
-		unsigned int end;
-
-		start = sp->start[field];
-		end = start + sp->count[field] - 1;
-
-		if (0 == par->first[field]
-		    || 0 == par->last[field]) {
-			/* No data on this field. */
-			continue;
-		}
-
-		if (0 == sp->count[field]) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Service 0x%08x (%s) requires "
-					"data from field %u",
-					par->id, par->label, field + 1);
-			return FALSE;
-		}
-
-		if (strict <= 0 || 0 == sp->start[field])
-			continue;
-
-		if (1 == strict && par->first[field] > par->last[field]) {
-			/* May succeed if not all scanning lines
-			   available for the service are actually used. */
-			continue;
-		}
-		
-		if (start > par->first[field]
-		    || end < par->last[field]) {
-			vbi_log_printf (log_fn, log_user_data,
-					VBI_LOG_NOTICE, __FUNCTION__,
-					"Service 0x%08x (%s) requires "
-					"lines %u-%u, have %u-%u.",
-					par->id, par->label,
-					par->first[field],
-					par->last[field],
-					start, end);
-			return FALSE;
-		}
-	}
-
-	return TRUE;
-}
-
-/* Attn: strict must be int for compatibility with libzvbi 0.2 (-1 == 0) */
-vbi_service_set
-vbi_sampling_par_check_services	(const vbi_sampling_par *sp,
-				 vbi_service_set	services,
-				 int			strict,
-				 vbi_log_fn *		log_fn,
-				 void *			log_user_data)
-{
-	const _vbi_service_par *par;
-	vbi_service_set rservices;
-
-	assert (NULL != sp);
-
-	rservices = 0;
-
-	for (par = _vbi_service_table; par->id; ++par) {
-		if (0 == (par->id & services))
-			continue;
-
-		if (_vbi_sampling_par_check_service (sp, par, strict,
-						     log_fn, log_user_data))
-			rservices |= par->id;
-	}
-
-	return rservices;
-}
-
-vbi_service_set
-vbi_sampling_par_from_services	(vbi_sampling_par *	sp,
-				 unsigned int *		max_rate,
-				 vbi_videostd_set	videostd_set,
-				 vbi_service_set	services,
-				 vbi_log_fn *		log_fn,
-				 void *			log_user_data)
-{
-	const _vbi_service_par *par;
-	vbi_service_set rservices;
-	unsigned int rate;
-
-	assert (NULL != sp);
-
-	if (0 != videostd_set) {
-		if (0 == (VBI_VIDEOSTD_SET_ALL & videostd_set)
-		    || ((VBI_VIDEOSTD_SET_525_60 & videostd_set)
-			&& (VBI_VIDEOSTD_SET_625_50 & videostd_set))) {
-			sp_log (VBI_LOG_WARNING,
-				"Ambiguous videostd_set 0x%08" PRIx64,
-				videostd_set);
-			/* CLEAR (*sp); */
-			return 0;
-		}
-	}
-
-	sp->sampling_format	= VBI_PIXFMT_YUV420; /* VBI_PIXFMT_Y8 */
-	sp->sampling_rate	= 27000000;		/* ITU-R BT.601 */
-	sp->bytes_per_line	= 0;
-	sp->offset		= (int)(64e-6 * sp->sampling_rate);
-	sp->start[0]		= 30000;
-	sp->count[0]		= 0;
-	sp->start[1]		= 30000;
-	sp->count[1]		= 0;
-	sp->interlaced		= FALSE;
-	sp->synchronous		= TRUE;
-
-	rservices = 0;
-	rate = 0;
-
-	for (par = _vbi_service_table; par->id; ++par) {
-		double margin;
-		double signal;
-		int offset;
-		unsigned int samples;
-		unsigned int i;
-
-		if (0 == (par->id & services))
-			continue;
-
-		if (0 == videostd_set) {
-			vbi_videostd_set set;
-
-			set = par->videostd_set | videostd_set;
-
-			if (0 == (VBI_VIDEOSTD_SET_525_60 & set)
-			    || 0 == (VBI_VIDEOSTD_SET_625_50 & set))
-				videostd_set = set;
-		}
-
-		if (VBI_VIDEOSTD_SET_525_60 & videostd_set)
-			margin = 1.0e-6;
-		else
-			margin = 2.0e-6;
-
-		if (0 == (par->videostd_set & videostd_set)) {
-			sp_log (VBI_LOG_NOTICE,
-				"Service 0x%08x (%s) requires "
-				"videostd_set 0x%" PRIx64 ", "
-				"have 0x%" PRIx64,
-				par->id, par->label,
-				par->videostd_set, videostd_set);
-			continue;
-		}
-
-		rate = MAX (rate, par->cri_rate);
-		rate = MAX (rate, par->bit_rate);
-
-		signal = par->cri_bits / (double) par->cri_rate
-			+ ((par->frc_bits + par->payload) / (double) par->bit_rate);
-
-		offset = (int)((par->offset / 1e9) * sp->sampling_rate);
-		samples = (int)((signal + 1.0e-6) * sp->sampling_rate);
-
-		sp->offset = MIN (sp->offset, offset);
-
-		/* Note bytes_per_sample is 1. */
-		sp->bytes_per_line =
-			MAX ((unsigned int) sp->bytes_per_line + sp->offset,
-			     samples + offset) - sp->offset;
-
-		for (i = 0; i < 2; ++i)
-			if (par->first[i] > 0
-			    && par->last[i] > 0) {
-				unsigned int start;
-				unsigned int end;
-
-				start = (unsigned int) sp->start[i];
-				sp->start[i] = MIN (start, par->first[i]);
-
-				end = sp->start[i] + sp->count[i];
-				sp->count[i] = MAX (end, par->last[i] + 1)
-					- sp->start[i];
-			}
-
-		rservices |= par->id;
-	}
-
-	if (0 == rservices) {
-		/* CLEAR (*sp); */
-		return 0;
-	}
-
-	if (videostd_set & VBI_VIDEOSTD_SET_525_60)
-		sp->scanning = 525;
-	else
-		sp->scanning = 625;
-
-	if (0 == sp->count[1]) {
-		sp->start[1] = 0;
-
-		if (0 == sp->count[0]) {
-			sp->start[0] = 0;
-			sp->offset = 0;
-		}
-	} else if (0 == sp->count[0]) {
-		sp->start[0] = 0;
-	}
-
-	if (max_rate)
-		*max_rate = rate;
-
-	return rservices;
-}
-
 /**
  * $param rd Pointer to vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
@@ -1060,10 +648,10 @@ vbi3_raw_decoder_reset		(vbi3_raw_decoder *	rd)
 {
 	assert (NULL != rd);
 
-	if (rd->pattern)
-		free (rd->pattern);
-
-	rd->pattern = NULL;
+	if (rd->pattern) {
+		vbi_free (rd->pattern);
+		rd->pattern = NULL;
+	}
 
 	rd->services = 0;
 	rd->n_jobs = 0;
@@ -1252,6 +840,7 @@ lines_containing_data		(unsigned int		start[2],
 
 	start[0] = 0;
 	start[1] = sp->count[0];
+
 	count[0] = sp->count[0];
 	count[1] = sp->count[1];
 
@@ -1274,17 +863,24 @@ lines_containing_data		(unsigned int		start[2],
 		first = sp->start[field];
 		last = first + sp->count[field] - 1;
 
-		if (first > 0) {
+		if (first > 0 && sp->count[field] > 0) {
+			assert (par->first[field] <= par->last[field]);
+
+			if ((unsigned int) par->first[field] > last
+			    || (unsigned int) par->last[field] < first)
+				continue;
+
 			first = MAX (first, (unsigned int) par->first[field]);
+			last = MIN ((unsigned int) par->last[field], last);
+
 			start[field] += first - sp->start[field];
-			last = MIN (last, (unsigned int) par->last[field]);
-			count[field] = last - first + 1;
+			count[field] = last + 1 - first;
 		}
 	}
 }
 
 /**
- * $param rd Pointer to vbi_raw_decoder object allocated with
+ * $param rd Pointer to vbi3_raw_decoder object allocated with
  *   vbi3_raw_decoder_new().
  * $param services Set of data services.
  * $param strict A value of 0, 1 or 2 requests loose, reliable or strict
@@ -1317,35 +913,40 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	services &= ~(VBI_SLICED_VBI_525 | VBI_SLICED_VBI_625);
 
 	if (rd->services & services) {
-		rd_log (VBI_LOG_INFO,
-			"Already decoding services 0x%08x.",
-			rd->services & services);
+		info (&rd->log,
+		      "Already decoding services 0x%08x.",
+		      rd->services & services);
 		services &= ~rd->services;
 	}
 
 	if (0 == services) {
-		rd_log (VBI_LOG_INFO, "No services to add.");
+		info (&rd->log, "No services to add.");
 		return rd->services;
 	}
 
 	if (!rd->pattern) {
 		unsigned int scan_lines;
 		unsigned int scan_ways;
+		unsigned int size;
 
 		scan_lines = rd->sampling.count[0] + rd->sampling.count[1];
 		scan_ways = scan_lines * _VBI3_RAW_DECODER_MAX_WAYS;
 
-		rd->pattern = (int8_t *)
-			malloc (scan_ways * sizeof (rd->pattern[0]));
-		if (!rd->pattern) {
-			rd_log (VBI_LOG_ERR, "Out of memory.");
+		size = scan_ways * sizeof (rd->pattern[0]);
+		rd->pattern = (int8_t *) vbi_malloc (size);
+		if (NULL == rd->pattern) {
+			error (&rd->log, "Out of memory.");
 			return rd->services;
 		}
 
 		memset (rd->pattern, 0, scan_ways * sizeof (rd->pattern[0]));
 	}
 
+#if 2 == VBI_VERSION_MINOR
 	if (525 == rd->sampling.scanning) {
+#else
+	if (VBI3_VIDEOSTD_SET_525_60 & rd->sampling.videostd_set) {
+#endif
 		min_offset = 7.9e-6;
 	} else {
 		min_offset = 8.0e-6;
@@ -1383,11 +984,11 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 		}
 
 		if (j >= _VBI3_RAW_DECODER_MAX_JOBS) {
-			rd_log (VBI_LOG_ERR,
-				"Set 0x%08x exceeds number of "
-				"simultaneously decodable "
-				"services (%u).",
-				services, _VBI3_RAW_DECODER_MAX_WAYS);
+			error (&rd->log,
+			       "Set 0x%08x exceeds number of "
+			       "simultaneously decodable "
+			       "services (%u).",
+			       services, _VBI3_RAW_DECODER_MAX_WAYS);
 			break;
 		} else if (j >= rd->n_jobs) {
 			job->id = 0;
@@ -1396,16 +997,18 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 
 		sp = &rd->sampling;
 
-		if (!_vbi_sampling_par_check_service
-		    (sp, par, strict, rd->log_fn, rd->log_user_data))
+		if (!_vbi_sampling_par_check_services_log (sp,
+							   par->id,
+							   strict,
+							   &rd->log))
 			continue;
 
 
 		sample_offset = 0;
 
 		/* Skip color burst. */
-		if (0
-		    && sp->offset > 0 && strict > 0) {
+		/* Offsets aren't that reliable, sigh. */
+		if (0 && sp->offset > 0 && strict > 0) {
 			double offset;
 
 			offset = sp->offset / (double) sp->sampling_rate;
@@ -1422,36 +1025,48 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 			cri_end = ~0;
 		}
 
+#if 2 == VBI_VERSION_MINOR
 		samples_per_line = sp->bytes_per_line
-			/ VBI_PIXFMT_BPP (sp->sampling_format);
+			/ VBI_PIXFMT_BPP (sp->sp_sample_format);
+#else
+		samples_per_line = sp->samples_per_line;
+#endif
 
-		if (!_vbi3_bit_slicer_init (&job->slicer,
-					   sp->sampling_format,
-					   sp->sampling_rate,
-					   sample_offset,
-					   samples_per_line,
-					   par->cri_frc >> par->frc_bits,
-					   par->cri_frc_mask >> par->frc_bits,
-					   par->cri_bits,
-					   par->cri_rate,
-					   cri_end,
-					   (par->cri_frc
-					    & ((1U << par->frc_bits) - 1)),
-					   par->frc_bits,
-					   par->payload,
-					   par->bit_rate,
-					   par->modulation)) {
+		if (!_vbi3_bit_slicer_init (&job->slicer)) {
 			assert (!"bit_slicer_init");
 		}
 
+		if (!vbi3_bit_slicer_set_params
+		    (&job->slicer,
+		     sp->sp_sample_format,
+		     sp->sampling_rate,
+		     sample_offset,
+		     samples_per_line,
+		     par->cri_frc >> par->frc_bits,
+		     par->cri_frc_mask >> par->frc_bits,
+		     par->cri_bits,
+		     par->cri_rate,
+		     cri_end,
+		     (par->cri_frc & ((1U << par->frc_bits) - 1)),
+		     par->frc_bits,
+		     par->payload,
+		     par->bit_rate,
+		     par->modulation)) {
+			assert (!"bit_slicer_set_params");
+		}
+
+		vbi3_bit_slicer_set_log_fn (&job->slicer,
+					    rd->log.mask,
+					    rd->log.fn,
+					    rd->log.user_data);
 
 		lines_containing_data (start, count, sp, par);
 
 		if (!add_job_to_pattern (rd, job - rd->jobs, start, count)) {
-			rd_log (VBI_LOG_ERR,
-				"Out of decoder pattern space for "
-				"service 0x%08x (%s).",
-				par->id, par->label);
+			error (&rd->log,
+			       "Out of decoder pattern space for "
+			       "service 0x%08x (%s).",
+			       par->id, par->label);
 			continue;
 		}
 
@@ -1466,6 +1081,88 @@ vbi3_raw_decoder_add_services	(vbi3_raw_decoder *	rd,
 	return rd->services;
 }
 
+vbi_bool
+vbi3_raw_decoder_sampling_point	(vbi3_raw_decoder *	rd,
+				 vbi3_bit_slicer_point *point,
+				 unsigned int		row,
+				 unsigned int		nth_bit)
+{
+	assert (NULL != rd);
+	assert (NULL != point);
+
+	if (row >= rd->n_sp_lines)
+		return FALSE;
+
+	if (nth_bit >= rd->sp_lines[row].n_points)
+		return FALSE;
+
+	*point = rd->sp_lines[row].points[nth_bit];
+
+	return TRUE;
+}
+
+vbi_bool
+vbi3_raw_decoder_debug		(vbi3_raw_decoder *	rd,
+				 vbi_bool		enable)
+{
+	_vbi3_raw_decoder_sp_line *sp_lines;
+	unsigned int n_lines;
+	vbi_bool r;
+
+	assert (NULL != rd);
+
+	sp_lines = NULL;
+	r = TRUE;
+
+	rd->debug = !!enable;
+
+	n_lines = 0;
+	if (enable) {
+		n_lines = rd->sampling.count[0] + rd->sampling.count[1];
+	}
+
+	switch (rd->sampling.sp_sample_format) {
+#if 3 == VBI_VERSION_MINOR
+	case VBI_PIXFMT_YUV444:
+	case VBI_PIXFMT_YVU444:
+	case VBI_PIXFMT_YUV422:
+	case VBI_PIXFMT_YVU422:
+	case VBI_PIXFMT_YUV411:
+	case VBI_PIXFMT_YVU411:
+	case VBI_PIXFMT_YVU420:
+	case VBI_PIXFMT_YUV410:
+	case VBI_PIXFMT_YVU410:
+	case VBI_PIXFMT_Y8:
+	case VBI_PIXFMT_UNKNOWN:
+#endif
+	case VBI_PIXFMT_YUV420:
+		break;
+
+	default:
+		/* Not implemented. */
+		n_lines = 0;
+		r = FALSE;
+		break;
+	}
+
+	if (rd->n_sp_lines == n_lines)
+		return r;
+
+	vbi_free (rd->sp_lines);
+	rd->sp_lines = NULL;
+	rd->n_sp_lines = 0;
+
+	if (n_lines > 0) {
+		rd->sp_lines = calloc (n_lines, sizeof (*rd->sp_lines));
+		if (NULL == rd->sp_lines)
+			return FALSE;
+
+		rd->n_sp_lines = n_lines;
+	}
+
+	return r;
+}
+
 vbi_service_set
 vbi3_raw_decoder_services	(vbi3_raw_decoder *	rd)
 {
@@ -1506,13 +1203,16 @@ vbi3_raw_decoder_set_sampling_par
 
 	vbi3_raw_decoder_reset (rd);
 
-	if (!_vbi_sampling_par_valid (sp, rd->log_fn, rd->log_user_data)) {
+	if (!_vbi_sampling_par_valid_log (sp, &rd->log)) {
 		CLEAR (rd->sampling);
 		return 0;
 	}
 
 	rd->sampling = *sp;
 
+	/* Error ignored. */
+	vbi3_raw_decoder_debug (rd, rd->debug);
+
 	return vbi3_raw_decoder_add_services (rd, services, strict);
 }
 
@@ -1538,34 +1238,43 @@ void
 vbi3_raw_decoder_set_log_fn	(vbi3_raw_decoder *	rd,
 				 vbi_log_fn *		log_fn,
 				 void *			user_data,
-				 vbi_log_level		max_level)
+				 vbi_log_mask		mask)
 {
+	unsigned int i;
+
 	assert (NULL != rd);
 
 	if (NULL == log_fn)
-		max_level = 0;
+		mask = 0;
 
-	rd->log_fn = log_fn;
-	rd->log_user_data = user_data;
-	rd->log_max_level = max_level;
+	rd->log.mask = mask;
+	rd->log.fn = log_fn;
+	rd->log.user_data = user_data;
+
+	for (i = 0; i < _VBI3_RAW_DECODER_MAX_JOBS; ++i) {
+		vbi3_bit_slicer_set_log_fn (&rd->jobs[i].slicer,
+					    mask, log_fn, user_data);
+	}
 }
 
 /**
- * $internal
+ * @internal
  *
- * Free all resources associated with $a rd.
+ * Free all resources associated with @a rd.
  */
 void
 _vbi3_raw_decoder_destroy	(vbi3_raw_decoder *	rd)
 {
 	vbi3_raw_decoder_reset (rd);
 
+	vbi3_raw_decoder_debug (rd, FALSE);
+
 	/* Make unusable. */
 	CLEAR (*rd);
 }
 
 /**
- * $internal
+ * @internal
  * 
  * See vbi3_raw_decoder_new().
  */
@@ -1577,16 +1286,8 @@ _vbi3_raw_decoder_init		(vbi3_raw_decoder *	rd,
 
 	vbi3_raw_decoder_reset (rd);
 
-	if (RAW_DECODER_LOG) {
-		vbi3_raw_decoder_set_log_fn (rd,
-					     vbi_log_on_stderr,
-					     /* user_data */ NULL,
-					     /* max_level */ VBI_LOG_INFO);
-	}
-
 	if (NULL != sp) {
-		if (!_vbi_sampling_par_valid (sp, rd->log_fn,
-					      rd->log_user_data))
+		if (!_vbi_sampling_par_valid_log (sp, &rd->log))
 			return FALSE;
 
 		rd->sampling = *sp;
@@ -1609,7 +1310,7 @@ vbi3_raw_decoder_delete		(vbi3_raw_decoder *	rd)
 
 	_vbi3_raw_decoder_destroy (rd);
 
-	free (rd);
+	vbi_free (rd);
 }
 
 /**
@@ -1624,7 +1325,7 @@ vbi3_raw_decoder_delete		(vbi3_raw_decoder *	rd)
  *
  * $returns
  * NULL when out of memory or the sampling parameters are invalid,
- * Otherwise a pointer to an opaque vbi_raw_decoder object which must
+ * Otherwise a pointer to an opaque vbi3_raw_decoder object which must
  * be deleted with vbi3_raw_decoder_delete() when done.
  */
 vbi3_raw_decoder *
@@ -1632,14 +1333,23 @@ vbi3_raw_decoder_new		(const vbi_sampling_par *sp)
 {
 	vbi3_raw_decoder *rd;
 
-	rd = malloc (sizeof (*rd));
-	if (NULL == rd)
+	rd = vbi_malloc (sizeof (*rd));
+	if (NULL == rd) {
+		errno = ENOMEM;
 		return NULL;
+	}
 
 	if (!_vbi3_raw_decoder_init (rd, sp)) {
-		free (rd);
+		vbi_free (rd);
 		rd = NULL;
 	}
 
 	return rd;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
