@@ -1,33 +1,34 @@
 /*
- *  libzvbi - Extended Data Service demultiplexer
+ *  libzvbi -- Extended Data Service demultiplexer
  *
  *  Copyright (C) 2000-2005 Michael H. Schimek
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Library General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this library; if not, write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
+ *  Boston, MA  02110-1301  USA.
  */
 
-/* $Id: xds_demux.c,v 1.5 2005/10/04 10:06:33 mschimek Exp $ */
+/* $Id: xds_demux.c,v 1.11 2008/02/19 00:35:23 mschimek Exp $ */
 
-#include "../site_def.h"
-#include "../config.h"
+#include "site_def.h"
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
-#include <assert.h>
-#include <stdlib.h>		/* malloc() */
-#include <string.h>		/* memcpy() */
-#include "hamm.h"		/* vbi_ipar8() */
 #include "misc.h"		/* vbi_log_printf() */
+#include "hamm.h"		/* vbi_ipar8() */
 #include "tables.h"		/* vbi_rating/prog_type_string() */
 #include "xds_demux.h"
 
@@ -61,7 +62,7 @@ xdump				(const vbi_xds_packet *	xp,
 	fputs (" '", fp);
 
 	for (i = 0; i < xp->buffer_size; ++i)
-		fputc (vbi_printable (xp->buffer[i]), fp);
+		fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 
 	fputc ('\'', fp);
 }
@@ -454,13 +455,13 @@ _vbi_xds_packet_dump		(const vbi_xds_packet *	xp,
 
 			fprintf (fp, ", call letters: ");
 			for (i = 8; i < 12; ++i) {
-				fputc (vbi_printable (xp->buffer[i]), fp);
+				fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 			}
 
 			/* 02 to 69 or 0x20 0x20 */
 			fprintf (fp, ", channel: ");
 			for (i = 12; i < 14; ++i) {
-				fputc (vbi_printable (xp->buffer[i]), fp);
+				fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 			}
 
 			fputc (')', fp);
@@ -755,19 +756,19 @@ _vbi_xds_packet_dump		(const vbi_xds_packet *	xp,
 
 			fprintf (fp, " (event category: ");
 			for (i = 0; i < 3; ++i) {
-				fputc (vbi_printable (xp->buffer[i]), fp);
+				fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 			}
 
 			/* 3 digit FIPS number. */
 			fprintf (fp, ", state: ");
 			for (i = 3; i < 6; ++i) {
-				fputc (vbi_printable (xp->buffer[i]), fp);
+				fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 			}
 
 			/* 3 digit FIPS number. */
 			fprintf (fp, ", county: ");
 			for (i = 6; i < 9; ++i) {
-				fputc (vbi_printable (xp->buffer[i]), fp);
+				fputc (_vbi_to_ascii (xp->buffer[i]), fp);
 			}
 
 			/* 2 digit number of quarter hours */
@@ -841,7 +842,7 @@ vbi_xds_demux_reset		(vbi_xds_demux *	xd)
  * You should feed only data from NTSC line 284.
  *
  * @returns
- * FALSE if the buffer contained parity errors.
+ * @c FALSE if the buffer contained parity errors.
  *
  * @since 0.2.16
  */
@@ -1006,6 +1007,50 @@ vbi_xds_demux_feed		(vbi_xds_demux *	xd,
 	return r;
 }
 
+/**
+ * @param xd XDS demultiplexer context allocated with vbi_xds_demux_new().
+ * @param sliced Sliced VBI data.
+ * @param n_lines Number of lines in the @a sliced array.
+ *
+ * This function works like vbi_xds_demux_feed() but operates on sliced
+ * VBI data and filters out @c VBI_SLICED_CAPTION_525 on NTSC line 284.
+ *
+ * @returns
+ * @c FALSE if any of the Caption lines contained parity errors.
+ *
+ * @since 0.2.26
+ */
+vbi_bool
+vbi_xds_demux_feed_frame	(vbi_xds_demux *	xd,
+				 const vbi_sliced *	sliced,
+				 unsigned int		n_lines)
+{
+	const vbi_sliced *end;
+
+	assert (NULL != xd);
+	assert (NULL != sliced);
+
+	for (end = sliced + n_lines; sliced < end; ++sliced) {
+		switch (sliced->id) {
+		case VBI_SLICED_CAPTION_525:
+		case VBI_SLICED_CAPTION_525_F2:
+			if (284 != sliced->line
+			    && 0 != sliced->line)
+				continue;
+
+			if (!vbi_xds_demux_feed (xd, sliced->data))
+				return FALSE;
+
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	return TRUE;
+}
+
 #if 0 /* ideas */
 const vbi_xds_packet *
 vbi_xds_demux_get_packet	(vbi_xds_demux *	xd,
@@ -1106,3 +1151,10 @@ vbi_xds_demux_new		(vbi_xds_demux_cb *	callback,
 
 	return xd;
 }
+
+/*
+Local variables:
+c-set-style: K&R
+c-basic-offset: 8
+End:
+*/
