@@ -1,8 +1,8 @@
 /*
- *  zvbi-decode -- decode sliced VBI data using low-level
+ *  zvbi-decode -- Decode sliced VBI data using low-level
  *		   libzvbi functions
  *
- *  Copyright (C) 2004, 2006 Michael H. Schimek
+ *  Copyright (C) 2004, 2006, 2007 Michael H. Schimek
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,10 +16,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
-/* $Id: decode.c,v 1.12 2006/05/08 16:49:23 mschimek Exp $ */
+/* $Id: decode.c,v 1.36 2009/12/14 23:43:52 mschimek Exp $ */
+
+/* For libzvbi version 0.2.x / 0.3.x. */
+
+#undef NDEBUG
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -40,297 +45,78 @@
 
 #include "src/version.h"
 #if 2 == VBI_VERSION_MINOR
-#  include "src/libzvbi.h"
-#  include "sliced.h"		/* sliced data from file */
-#  define HAVE_VBI_PFC_DEMUX 1 /* XXX port me */
-#else /* 0.3 */
+#  include "src/bcd.h"
+#  include "src/cc608_decoder.h"
+#  include "src/conv.h"
+#  include "src/dvb_demux.h"
+#  include "src/hamm.h"
+#  include "src/idl_demux.h"
+#  include "src/lang.h"
+#  include "src/packet-830.h"
+#  include "src/pfc_demux.h"
+#  include "src/vps.h"
+#  include "src/xds_demux.h"
+#elif 3 == VBI_VERSION_MINOR
 #  include "src/zvbi.h"
-#  include "src/misc.h"		/* vbi_printable() */
-   /* XXX update me */
-#  define vbi_dvb_pes_demux_new _vbi_dvb_pes_demux_new
-#  define vbi_dvb_demux_cor _vbi_dvb_demux_cor
-#  define vbi_dvb_demux_delete _vbi_dvb_demux_delete
+#  include "src/misc.h"		/* _vbi_to_ascii() */
+#else
+#  error VBI_VERSION_MINOR == ?
 #endif
 
+#include "sliced.h"
+
+#undef _
 #define _(x) x /* i18n TODO */
 
 /* Will be installed one day. */
 #define PROGRAM_NAME "zvbi-decode"
 
-#ifndef PRId64
-#  define PRId64 "lld"
-#endif
+static const char *		option_in_file_name;
+static enum file_format		option_in_file_format;
+static unsigned int		option_in_ts_pid;
 
-static vbi_bool			source_is_pes; /* ATSC/DVB */
-
-static vbi_pgno			option_pfc_pgno;
+static vbi_pgno		option_pfc_pgno;
 static unsigned int		option_pfc_stream;
 
-static vbi_bool			option_decode_ttx;
-static vbi_bool			option_decode_8301;
-static vbi_bool			option_decode_8302;
-static vbi_bool			option_decode_caption;
-static vbi_bool			option_decode_xds;
-static vbi_bool			option_decode_idl;
-static vbi_bool			option_decode_vps;
-static vbi_bool			option_decode_vps_other;
-static vbi_bool			option_decode_wss;
+static vbi_bool		option_decode_ttx;
+static vbi_bool		option_decode_8301;
+static vbi_bool		option_decode_8302;
+static vbi_bool		option_decode_caption;
+static vbi_bool		option_decode_xds;
+static vbi_bool		option_decode_idl;
+static vbi_bool		option_decode_vps;
+static vbi_bool		option_decode_vps_other;
+static vbi_bool		option_decode_wss;
 
-static vbi_bool			option_dump_network;
-static vbi_bool			option_dump_hex;
-static vbi_bool			option_dump_bin;
-static vbi_bool			option_dump_time;
+static vbi_bool		option_dump_network;
+static vbi_bool		option_dump_hex;
+static vbi_bool		option_dump_bin;
+static vbi_bool		option_dump_time;
+static double			option_metronome_tick;
 
-static vbi_pgno			option_pfc_pgno	= 0;
+static vbi_pgno		option_pfc_pgno	= 0;
 static unsigned int		option_pfc_stream = 0;
 
 static unsigned int		option_idl_channel = 0;
 static unsigned int		option_idl_address = 0;
 
-/* Demultiplexers. */
+static struct stream *		rst;
 
-#ifdef HAVE_VBI_PFC_DEMUX
 static vbi_pfc_demux *		pfc;
-#endif
-static vbi_dvb_demux *		dvb;
 static vbi_idl_demux *		idl;
 static vbi_xds_demux *		xds;
 
-#ifndef HAVE_PROGRAM_INVOCATION_NAME
-static char *			program_invocation_name;
-static char *			program_invocation_short_name;
-#endif
-
 extern void
 _vbi_pfc_block_dump		(const vbi_pfc_block *	pb,
 				 FILE *			fp,
 				 vbi_bool		binary);
 
-static int
-vbi_printable			(int			c)
-{
-	if (c < 0)
-		return '?';
-
-	c &= 0x7F;
-
-	if (c < 0x20 || c >= 0x7F)
-		return '.';
-
-	return c;
-}
-
-static void
-error_exit			(const char *		template,
-				 ...)
-{
-	va_list ap;
-
-	fprintf (stderr, "%s: ", program_invocation_short_name);
-
-	va_start (ap, template);
-	vfprintf (stderr, template, ap);
-	va_end (ap);         
-
-	fputc ('\n', stderr);
-
-	exit (EXIT_FAILURE);
-}
-
-static void
-no_mem_exit			(void)
-{
-	error_exit (_("Out of memory."));
-}
-
-static void
-caption_command			(unsigned int		line,
-				 unsigned int		c1,
-				 unsigned int		c2)
-{
-	unsigned int ch;
-	unsigned int a7;
-	unsigned int f;
-	unsigned int b7;
-	unsigned int u;
-
-	printf ("CC line=%3u cmd 0x%02x 0x%02x ", line, c1, c2);
-
-	if (0 == c1) {
-		printf ("null\n");
-		return;
-	}
-
-	/* Some common bit groups. */
-
-	ch = (c1 >> 3) & 1; /* channel (language) */
-	a7 = c1 & 7;
-	f = c1 & 1; /* field */
-	b7 = (c2 >> 1) & 7;
-	u = c2 & 1; /* underline */
-
-	if (c2 >= 0x40) {
-		static const int row [16] = {
-			/* 0 */ 10,			/* 0x1040 */
-			/* 1 */ -1,			/* unassigned */
-			/* 2 */ 0, 1, 2, 3,		/* 0x1140 ... 0x1260 */
-			/* 6 */ 11, 12, 13, 14,		/* 0x1340 ... 0x1460 */
-			/* 10 */ 4, 5, 6, 7, 8, 9	/* 0x1540 ... 0x1760 */
-		};
-		unsigned int rrrr;
-
-		/* Preamble Address Codes -- 001 crrr  1ri xxxu */
-  
-		rrrr = a7 * 2 + ((c2 >> 5) & 1);
-
-		if (c2 & 0x10)
-			printf ("PAC ch=%u row=%u column=%u u=%u\n",
-				ch, row[rrrr], b7 * 4, u);
-		else
-			printf ("PAC ch=%u row=%u color=%u u=%u\n",
-				ch, row[rrrr], b7, u);
-		return;
-	}
-
-	switch (c1 & 7) {
-	case 0:
-		if (c2 & 0x10)
-			break;
-
-		printf ("bkg. color ch=%u color=%u translucent=%u\n",
-			ch, b7, u);
-		return;
-
-	case 1:
-		if (c2 & 0x10) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-			uint16_t ucs2_str[1];
-
-			c2 &= 15;
-
-			printf ("special character ch=%u %u='",
-				ch, c2);
-			/* All caption characters are representable
-			   in UTF-8, but not necessarily in ASCII. */
-			ucs2_str[0] = vbi_caption_unicode (c2);
-			vbi_fputs_locale_ucs2 (stdout, ucs2_str, 1);
-			puts ("'");
-#else
-			printf ("special character ch=%u %u\n",
-				ch, c2 & 15);
-#endif
-		} else {
-			printf ("mid-row ch=%u color=%u u=%u\n", ch, b7, u);
-		}
-
-		return;
-
-	case 2: /* ? */
-	case 3: /* ? */
-		break;
-
-	case 4:
-	case 5:
-		if (c2 & 0x10)
-			break;
-
-		switch (c2 & 15) {
-		case 0:	
-			printf ("resume caption ch=%u f=%u\n", ch, f);
-			return;
-
-		case 1:
-			printf ("backspace ch=%u f=%u\n", ch, f);
-			return;
-
-		case 2:
-			printf ("alarm off ch=%u f=%u\n", ch, f);
-			return;
-
-		case 3:
-			printf ("alarm on ch=%u f=%u\n", ch, f);
-			return;
-
-		case 4:
-			printf ("delete to end of row ch=%u f=%u\n", ch, f);
-			return;
-
-		case 5:
-		case 6:
-		case 7:
-			printf ("roll-up caption ch=%u f=%u rows=%u\n",
-				ch, f, (c2 & 7) - 3);
-			return;
-
-		case 8:
-			printf ("flash on ch=%u f=%u\n", ch, f);
-			return;
-
-		case 9:
-			printf ("resume direct ch=%u f=%u\n", ch, f);
-			return;
-
-		case 10:
-			printf ("text restart ch=%u f=%u\n", ch, f);
-			return;
-
-		case 11:
-			printf ("resume text ch=%u f=%u\n", ch, f);
-			return;
-
-		case 12:
-			printf ("erase displayed ch=%u f=%u\n", ch, f);
-			return;
-
-		case 13:
-			printf ("carriage return ch=%u f=%u\n", ch, f);
-			return;
-
-		case 14:
-			printf ("erase non-displayed ch=%u f=%u\n", ch, f);
-			return;
-
-		case 15:
-			printf ("end of caption ch=%u f=%u\n", ch, f);
-			return;
-		}
-
-		break;
-
-	case 6: /* reserved */
-		break;
-
-	case 7:
-		switch (c2) {
-		case 0x21 ... 0x23:
-			printf ("tab ch=%u offs=%u\n", ch, c2 & 3);
-			return;
-
-		case 0x2D:
-			printf ("transp. bkg. ch=%u\n", ch);
-			return;
-
-		case 0x2E:
-		case 0x2F:
-			printf ("black bkg. ch=%u u=%u\n", ch, u);
-			return;
-
-		default: /* ? */
-			break;
-		}
-
-		break;
-	}
-
-	printf ("unknown\n");
-}
-
 static vbi_bool
 xds_cb				(vbi_xds_demux *	xd,
 				 const vbi_xds_packet *	xp,
 				 void *			user_data)
 {
-	xd = xd;
+	xd = xd; /* unused */
 	user_data = user_data;
 
 	_vbi_xds_packet_dump (xp, stdout);
@@ -344,58 +130,16 @@ caption				(const uint8_t		buffer[2],
 {
 	if (option_decode_xds && 284 == line) {
 		if (!vbi_xds_demux_feed (xds, buffer)) {
-			printf (_("Parity error in XDS data.\n"));
+			printf ("Parity error in XDS data.\n");
 		}
 	}
 
 	if (option_decode_caption
 	    && (21 == line || 284 == line /* NTSC */
-		|| 22 == line /* PAL */)) {
-		int c1;
-		int c2;
+		|| 22 == line /* PAL? */)) {
 
-		c1 = vbi_unpar8 (buffer[0]);
-		c2 = vbi_unpar8 (buffer[1]);
-
-		if ((c1 | c2) < 0) {
-			printf (_("Parity error in CC line=%u "
-				  " %s0x%02x %s0x%02x.\n"),
-				line,
-				(c1 < 0) ? ">" : "", buffer[0] & 0xFF,
-				(c2 < 0) ? ">" : "", buffer[1] & 0xFF);
-		} else if (c1 >= 0x20) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-			uint16_t ucs2_str[2];
-
-			printf ("CC line=%3u text 0x%02x 0x%02x '",
-				line, c1, c2);
-
-			/* All caption characters are representable
-			   in UTF-8, but not necessarily in ASCII. */
-			ucs2_str[0] = vbi_caption_unicode (c1);
-			if (c2 >= 0x20) {
-				ucs2_str[1] = vbi_caption_unicode (c2);
-				vbi_fputs_locale_ucs2 (stdout, ucs2_str, 2);
-			} else {
-				vbi_fputs_locale_ucs2 (stdout, ucs2_str, 1);
-			}
-
-			puts ("'");
-#else
-			printf ("CC line=%3u text 0x%02x 0x%02x '%c%c'\n",
-				line, c1, c2,
-				vbi_printable (c1),
-				vbi_printable (c2));
-#endif
-		} else if (0 == c1 || c1 >= 0x10) {
-			caption_command (line, c1, c2);
-		} else if (option_decode_xds) {
-			printf ("CC line=%3u cmd 0x%02x 0x%02x ", line, c1, c2);
-			if (0x0F == c1)
-				puts ("XDS packet end");
-			else
-				puts ("XDS packet start/continue");
-		}
+		printf ("CC line=%3u ", line);
+		_vbi_cc608_dump (stdout, buffer[0], buffer[1]);
 	}
 }
 
@@ -446,10 +190,10 @@ dump_bytes			(const uint8_t *	buffer,
 	putchar ('>');
 
 	for (j = 0; j < n_bytes; ++j) {
-		/* For Teletext: Not all characters are representable
+		/* Not all Teletext characters are representable
 		   in ASCII or even UTF-8, but at this stage we don't
 		   know the Teletext code page for a proper conversion. */
-		char c = vbi_printable (buffer[j]);
+		char c = _vbi_to_ascii (buffer[j]);
 
 		putchar (c);
 	}
@@ -457,8 +201,6 @@ dump_bytes			(const uint8_t *	buffer,
 	puts ("<");
 }
 
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-
 static void
 packet_8301			(const uint8_t		buffer[42],
 				 unsigned int		designation)
@@ -472,14 +214,14 @@ packet_8301			(const uint8_t		buffer[42],
 		return;
 
 	if (!vbi_decode_teletext_8301_cni (&cni, buffer)) {
-		printf (_("Error in Teletext "
-			  "packet 8/30 format 1 CNI.\n"));
+		printf ("Error in Teletext "
+			"packet 8/30 format 1 CNI.\n");
 		return;
 	}
 
 	if (!vbi_decode_teletext_8301_local_time (&time, &gmtoff, buffer)) {
-		printf (_("Error in Teletext "
-			  "packet 8/30 format 1 local time.\n"));
+		printf ("Error in Teletext "
+			"packet 8/30 format 1 local time.\n");
 		return;
 	}
 
@@ -492,8 +234,10 @@ packet_8301			(const uint8_t		buffer[42],
 		tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
 		tm.tm_hour, tm.tm_min, tm.tm_sec);
 
+#if 3 == VBI_VERSION_MINOR
 	if (0 != cni)
 		dump_cni (VBI_CNI_TYPE_8301, cni);
+#endif
 }
 
 static void
@@ -507,14 +251,14 @@ packet_8302			(const uint8_t		buffer[42],
 		return;
 
 	if (!vbi_decode_teletext_8302_cni (&cni, buffer)) {
-		printf (_("Error in Teletext "
-			  "packet 8/30 format 2 CNI.\n"));
+		printf ("Error in Teletext "
+			"packet 8/30 format 2 CNI.\n");
 		return;
 	}
 
 	if (!vbi_decode_teletext_8302_pdc (&pi, buffer)) {
-		printf (_("Error in Teletext "
-			  "packet 8/30 format 2 PDC data.\n"));
+		printf ("Error in Teletext "
+			"packet 8/30 format 2 PDC data.\n");
 		return;
 	}
 
@@ -524,16 +268,16 @@ packet_8302			(const uint8_t		buffer[42],
 
 	putchar ('\n');
 
+#if 3 == VBI_VERSION_MINOR
 	if (0 != pi.cni)
 		dump_cni (pi.cni_type, pi.cni);
+#endif
 }
 
-#endif /* 3 == VBI_VERSION_MINOR */
-
 static vbi_bool
 page_function_clear_cb		(vbi_pfc_demux *	dx,
-		                 const vbi_pfc_block *	block,
-				 void *			user_data)
+				 void *			user_data,
+		                 const vbi_pfc_block *	block)
 {
 	dx = dx; /* unused */
 	user_data = user_data;
@@ -579,7 +323,7 @@ packet_idl			(const uint8_t		buffer[42],
 
 	case 4:
 	case 12:
-		printf (_("(Low bit rate audio) "));
+		printf ("(Low bit rate audio) ");
 
 		dump_bytes (buffer, 42);
 
@@ -594,8 +338,8 @@ packet_idl			(const uint8_t		buffer[42],
 		pa |= vbi_unham8 (buffer[5]) << 8;
 
 		if (pa < 0) {
-			printf (_("Hamming error in Datavideo "
-				  "packet-address byte.\n"));
+			printf ("Hamming error in Datavideo "
+				"packet-address byte.\n");
 			return;
 		}
 
@@ -610,9 +354,10 @@ packet_idl			(const uint8_t		buffer[42],
 	case 10:
 	case 11:
 	case 15:
-		if ((ft = vbi_unham8 (buffer[2])) < 0) {
-			printf (_("Hamming error in IDL format "
-				  "A or B format-type byte.\n"));
+		ft = vbi_unham8 (buffer[2]);
+		if (ft < 0) {
+			printf ("Hamming error in IDL format "
+				"A or B format-type byte.\n");
 			return;
 		}
 
@@ -622,10 +367,11 @@ packet_idl			(const uint8_t		buffer[42],
 			int spa; /* service packet address */
 			unsigned int i;
 
-			if ((ial = vbi_unham8 (buffer[3])) < 0) {
-				printf (_("Hamming error in IDL format "
-					  "A interpretation-and-address-"
-					  "length byte.\n"));
+			ial = vbi_unham8 (buffer[3]);
+			if (ial < 0) {
+				printf ("Hamming error in IDL format "
+					"A interpretation-and-address-"
+					"length byte.\n");
 				return;
 			}
 
@@ -642,8 +388,8 @@ packet_idl			(const uint8_t		buffer[42],
 				spa |= vbi_unham8 (buffer[4 + i]) << (4 * i);
 
 			if (spa < 0) {
-				printf (_("Hamming error in IDL format "
-					  "A service-packet-address byte.\n"));
+				printf ("Hamming error in IDL format "
+					"A service-packet-address byte.\n");
 				return;
 			}
 
@@ -654,9 +400,10 @@ packet_idl			(const uint8_t		buffer[42],
 
 			an = (ft >> 2);
 
-			if ((ai = vbi_unham8 (buffer[3])) < 0) {
-				printf (_("Hamming error in IDL format "
-					  "B application-number byte.\n"));
+			ai = vbi_unham8 (buffer[3]);
+			if (ai < 0) {
+				printf ("Hamming error in IDL format "
+					"B application-number byte.\n");
 				return;
 			}
 
@@ -682,15 +429,21 @@ teletext			(const uint8_t		buffer[42],
 	unsigned int magazine;
 	unsigned int packet;
 
-#ifdef HAVE_VBI_PFC_DEMUX
 	if (NULL != pfc) {
 		if (!vbi_pfc_demux_feed (pfc, buffer)) {
-			printf (_("Error in Teletext "
-				  "PFC packet.\n"));
+			printf ("Error in Teletext "
+				"PFC packet.\n");
+			return;
+		}
+	}
+
+	if (NULL != idl) {
+		if (!vbi_idl_demux_feed (idl, buffer)) {
+			printf ("Error in Teletext "
+				"IDL packet.\n");
 			return;
 		}
 	}
-#endif
 
 	if (!(option_decode_ttx |
 	      option_decode_8301 |
@@ -700,8 +453,8 @@ teletext			(const uint8_t		buffer[42],
 
 	pmag = vbi_unham16p (buffer);
 	if (pmag < 0) {
-		printf (_("Hamming error in Teletext "
-			  "packet number.\n"));
+		printf ("Hamming error in Teletext "
+			"packet number.\n");
 		return;
 	}
 
@@ -716,22 +469,18 @@ teletext			(const uint8_t		buffer[42],
 
 		designation = vbi_unham8 (buffer[2]);
 		if (designation < 0 ) {
-			printf (_("Hamming error in Teletext "
-				  "packet 8/30 designation byte.\n"));
+			printf ("Hamming error in Teletext "
+				"packet 8/30 designation byte.\n");
 			return;
 		}
 
 		if (designation >= 0 && designation <= 1) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
 			packet_8301 (buffer, designation);
-#endif
 			return;
 		}
 
 		if (designation >= 2 && designation <= 3) {
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
 			packet_8302 (buffer, designation);
-#endif
 			return;
 		}
 	}
@@ -763,36 +512,35 @@ vps				(const uint8_t		buffer[13],
 {
 	if (option_decode_vps) {
 		unsigned int cni;
-#if 3 == VBI_VERSION_MINOR
+#if 1
 		vbi_program_id pi;
 #endif
 		if (option_dump_bin) {
 			printf ("VPS line=%3u ", line);
 			fwrite (buffer, 1, 13, stdout);
+			fflush (stdout);
 			return;
 		}
 
 		if (!vbi_decode_vps_cni (&cni, buffer)) {
-			printf (_("Error in VPS packet CNI.\n"));
+			printf ("Error in VPS packet CNI.\n");
 			return;
 		}
 
-#if 3 == VBI_VERSION_MINOR
 		if (!vbi_decode_vps_pdc (&pi, buffer)) {
-			printf (_("Error in VPS packet PDC data.\n"));
+			printf ("Error in VPS packet PDC data.\n");
 			return;
 		}
-		
+
 		printf ("VPS line=%3u ", line);
 
 		_vbi_program_id_dump (&pi, stdout);
 
 		putchar ('\n');
 
+#if 3 == VBI_VERSION_MINOR
 		if (0 != pi.cni)
 			dump_cni (pi.cni_type, pi.cni);
-#else
-		printf ("VPS line=%3u CNI=%x\n", line, cni);
 #endif
 	}
 
@@ -813,7 +561,7 @@ vps				(const uint8_t		buffer[13],
 			l[i] = 0;
 		}
 
-		label[i][l[i]] = vbi_printable (c);
+		label[i][l[i]] = _vbi_to_ascii (c);
 
 		l[i] = (l[i] + 1) % 16;
 		
@@ -822,7 +570,7 @@ vps				(const uint8_t		buffer[13],
 			"%02x %02x %02x %02x (\"%s\")\n",
 			line,
 			buffer[0], buffer[1],
-			c, vbi_printable (c),
+			c, _vbi_to_ascii (c),
 			buffer[2], buffer[3],
 			buffer[4], buffer[5], buffer[6], buffer[7],
 			pr_label[i]);
@@ -838,10 +586,12 @@ wss_625				(const uint8_t		buffer[2])
 		vbi_aspect_ratio ar;
 
 		if (!vbi_decode_wss_625 (&ar, buffer)) {
-			printf (_("Error in WSS packet.\n"));
+			printf ("Error in WSS packet.\n");
 			return;
 		}
 
+		fputs ("WSS ", stdout);
+
 		_vbi_aspect_ratio_dump (&ar, stdout);
 
 		putchar ('\n');
@@ -850,26 +600,44 @@ wss_625				(const uint8_t		buffer[2])
 
 #endif /* 3 == VBI_VERSION_MINOR */
 
-static void
-decode				(const vbi_sliced *	s,
+static vbi_bool
+decode_frame			(const vbi_sliced *	s,
 				 unsigned int		n_lines,
+				 const uint8_t *	raw,
+				 const vbi_sampling_par *sp,
 				 double			sample_time,
 				 int64_t		stream_time)
 {
+	static double metronome = 0.0;
 	static double last_sample_time = 0.0;
 	static int64_t last_stream_time = 0;
 
-	if (option_dump_time) {
+	raw = raw; /* unused */
+	sp = sp;
+
+	if (option_dump_time || option_metronome_tick > 0.0) {
 		/* Sample time: When we captured the data, in
 		   		seconds since 1970-01-01 (gettimeofday()).
-		   Stream time: For ATSC/DVB the Presentation TimeStamp.
+		   Stream time: For ATSC/DVB the Presentation Time Stamp.
 				For analog the frame number multiplied by
 				the nominal frame period (1/25 or
-				1001/30000 s). Both given in 90000 kHz units.
+				1001/30000 s). Both given in 90 kHz units.
 		   Note this isn't fully implemented yet. */
-		printf ("ST %f (%+f) PTS %" PRId64 " (%+" PRId64 ")\n",
-			sample_time, sample_time - last_sample_time,
-			stream_time, stream_time - last_stream_time);
+
+		if (option_metronome_tick > 0.0) {
+			printf ("ST %f (adv %+f, err %+f) PTS %"
+				PRId64 " (adv %+" PRId64 ", err %+f)\n",
+				sample_time, sample_time - last_sample_time,
+				sample_time - metronome,
+				stream_time, stream_time - last_stream_time,
+				(double) stream_time - metronome);
+
+			metronome += option_metronome_tick;
+		} else {
+			printf ("ST %f (%+f) PTS %" PRId64 " (%+" PRId64 ")\n",
+				sample_time, sample_time - last_sample_time,
+				stream_time, stream_time - last_stream_time);
+		}
 
 		last_sample_time = sample_time;
 		last_stream_time = stream_time;
@@ -910,173 +678,33 @@ decode				(const vbi_sliced *	s,
 		++s;
 		--n_lines;
 	}
+
+	return TRUE;
 }
 
-static void
-pes_mainloop			(void)
-{
-	uint8_t buffer[2048];
-
-	while (1 == fread (buffer, sizeof (buffer), 1, stdin)) {
-		const uint8_t *bp;
-		unsigned int left;
-
-		bp = buffer;
-		left = sizeof (buffer);
-
-		while (left > 0) {
-			vbi_sliced sliced[64];
-			unsigned int n_lines;
-			int64_t pts;
-
-			n_lines = vbi_dvb_demux_cor (dvb, sliced, 64,
-						     &pts, &bp, &left);
-			if (n_lines > 0)
-				decode (sliced, n_lines,
-					/* sample_time */ 0,
-					/* stream_time */ pts);
-		}
-	}
-
-	fprintf (stderr, _("\rEnd of stream\n"));
-}
-
-#if 3 == VBI_VERSION_MINOR /* XXX replace me, I'm redundant */
-
-static void
-old_mainloop			(void)
-{
-	double time;
-
-	time = 0.0;
-
-	for (;;) {
-		char buf[256];
-		double dt;
-		unsigned int n_items;
-		vbi_sliced sliced[40];
-		vbi_sliced *s;
-
-		if (ferror (stdin) || !fgets (buf, 255, stdin))
-			goto abort;
-
-		dt = strtod (buf, NULL);
-		n_items = fgetc (stdin);
-
-		assert (n_items < 40);
-
-		s = sliced;
-
-		while (n_items-- > 0) {
-			int index;
-
-			index = fgetc (stdin);
-
-			s->line = (fgetc (stdin)
-				   + 256 * fgetc (stdin)) & 0xFFF;
-
-			if (index < 0)
-				goto abort;
-
-			switch (index) {
-			case 0:
-				s->id = VBI_SLICED_TELETEXT_B;
-				fread (s->data, 1, 42, stdin);
-				break;
-
-			case 1:
-				s->id = VBI_SLICED_CAPTION_625; 
-				fread (s->data, 1, 2, stdin);
-				break; 
-
-			case 2:
-				s->id = VBI_SLICED_VPS;
-				fread (s->data, 1, 13, stdin);
-				break;
-
-			case 3:
-				s->id = VBI_SLICED_WSS_625; 
-				fread (s->data, 1, 2, stdin);
-				break;
-
-			case 4:
-				s->id = VBI_SLICED_WSS_CPR1204; 
-				fread (s->data, 1, 3, stdin);
-				break;
-
-			case 7:
-				s->id = VBI_SLICED_CAPTION_525; 
-				fread(s->data, 1, 2, stdin);
-				break;
-
-			default:
-				fprintf (stderr,
-					 "\nOops! Unknown data type %d "
-					 "in sample file\n", index);
-				exit (EXIT_FAILURE);
-			}
-
-			++s;
-		}
-
-		decode (sliced, s - sliced, time, 0);
-
-		if (feof (stdin) || ferror (stdin))
-			goto abort;
-
-		time += dt;
-	}
-
-	return;
-
-abort:
-	fprintf (stderr, "\rEnd of stream\n");
-}
-
-#else /* 2 == VBI_VERSION_MINOR */
-
-static void
-old_mainloop			(void)
-{
-	for (;;) {
-		vbi_sliced sliced[40];
-		double timestamp;
-		int n_lines;
-
-		n_lines = read_sliced (sliced, &timestamp,
-				       /* max_lines */ 40);
-		if (n_lines < 0)
-			break;
-
-		decode (sliced, n_lines,
-			/* sample_time */ timestamp,
-			/* stream_time */ 0);
-	}
-
-	fprintf (stderr, "\rEnd of stream\n");
-}
-
-#endif /* 2 == VBI_VERSION_MINOR */
-
 static void
 usage				(FILE *			fp)
 {
+	/* FIXME Supposed to be localized but we can't use #ifs
+	   within the _() macro. */
 	fprintf (fp, "\
-%s %s -- low-level VBI decoder\n\n\
-Copyright (C) 2004, 2006 Michael H. Schimek\n\
-This program is licensed under GPL 2 or later. NO WARRANTIES.\n\n\
-Usage: %s [options] < sliced VBI data\n\n\
+%s %s -- Low-level VBI decoder\n\n\
+Copyright (C) 2004, 2006, 2007 Michael H. Schimek\n\
+This program is licensed under GPLv2 or later. NO WARRANTIES.\n\n\
+Usage: %s [options] < sliced VBI data\n\
 -h | --help | --usage  Print this message and exit\n\
+-q | --quiet           Suppress progress and error messages\n\
 -V | --version         Print the program version and exit\n\
 Input options:\n\
--P | --pes             Source is a DVB PES stream [auto-detected]\n\
-Decoding options:\n"
-#if 3 == VBI_VERSION_MINOR /* XXX port me back */
-"-1 | --8301            Teletext packet 8/30 format 1 (local time)\n\
--2 | --8302            Teletext packet 8/30 format 2 (PDC)\n"
-#endif
-"-c | --cc              Closed Caption\n\
--i | --idl             Any Teletext IDL packets (M/30, M/31)\n\
+-i | --input name      Read the VBI data from this file instead of\n\
+                       standard input\n\
+-P | --pes             Source is a DVB PES stream\n\
+-T | --ts pid          Source is a DVB TS stream\n\
+Decoding options:\n\
+-1 | --8301            Teletext packet 8/30 format 1 (local time)\n\
+-2 | --8302            Teletext packet 8/30 format 2 (PDC)\n\
+-c | --cc              Closed Caption\n\
+-j | --idl             Any Teletext IDL packets (M/30, M/31)\n\
 -t | --ttx             Decode any Teletext packet\n\
 -v | --vps             Video Programming System (PDC)\n"
 #if 3 == VBI_VERSION_MINOR /* XXX port me back */
@@ -1087,14 +715,14 @@ Decoding options:\n"
                        -i     decode IDL packets\n\
                        -a     decode everything\n\
                        -a -i  everything except IDL\n\
--c | --idl-ch N\n\
--d | --idl-addr NNN\n\
-                       Decode Teletext IDL format A data from channel N,\n\
-                       service packet address NNN [0]\n\
+-l | --idl-ch N\n\
+-d | --idl-addr NNN    Decode Teletext IDL format A data from channel N,\n\
+                       service packet address NNN (default 0)\n\
 -r | --vps-other       Decode VPS data unrelated to PDC\n\
 -p | --pfc-pgno NNN\n\
 -s | --pfc-stream NN   Decode Teletext Page Function Clear data\n\
-                         from page NNN (for example 1DF), stream NN [0]\n\
+                       from page NNN (for example 1DF), stream NN\n\
+                       (default 0)\n\
 Modifying options:\n\
 -e | --hex             With -t dump packets in hex and ASCII,\n\
                          otherwise only ASCII\n\
@@ -1102,13 +730,15 @@ Modifying options:\n\
                          available information about the network\n\
 -b | --bin             With -t, -p, -v dump data in binary format\n\
                          instead of ASCII\n\
--T | --time            Dump capture timestamps\n\
+-m | --time            Dump capture timestamps\n\
+-M | --metronome tick  Compare timestamps against a metronome advancing\n\
+                       by tick seconds per frame\n\
 ",
 		 PROGRAM_NAME, VERSION, program_invocation_name);
 }
 
 static const char
-short_options [] = "12abcd:ehil:np:rs:tvwxPTV";
+short_options [] = "12abcd:ehi:jl:mnp:qrs:tvwxM:PT:V";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
@@ -1122,18 +752,22 @@ long_options [] = {
 	{ "hex",	no_argument,		NULL,		'e' },
 	{ "help",	no_argument,		NULL,		'h' },
 	{ "usage",	no_argument,		NULL,		'h' },
-	{ "idl",	no_argument,		NULL,		'i' },
+	{ "input",	required_argument,	NULL,		'i' },
+	{ "idl",	no_argument,		NULL,		'j' },
 	{ "idl-ch",	required_argument,	NULL,		'l' },
+	{ "time",	no_argument,		NULL,		'm' },
 	{ "network",	no_argument,		NULL,		'n' },
 	{ "pfc-pgno",	required_argument,	NULL,		'p' },
+	{ "quiet",	no_argument,		NULL,		'q' },
 	{ "vps-other",	no_argument,		NULL,		'r' },
 	{ "pfc-stream",	required_argument,	NULL,		's' },
 	{ "ttx",	no_argument,		NULL,		't' },
 	{ "vps",	no_argument,		NULL,		'v' },
 	{ "wss",	no_argument,		NULL,		'w' },
 	{ "xds",	no_argument,		NULL,		'x' },
+	{ "metronome",	required_argument,	NULL,		'M' },
 	{ "pes",	no_argument,		NULL,		'P' },
-	{ "time",	no_argument,		NULL,		'T' },
+	{ "ts",		required_argument,	NULL,		'T' },
 	{ "version",	no_argument,		NULL,		'V' },
 	{ NULL, 0, 0, 0 }
 };
@@ -1147,22 +781,9 @@ int
 main				(int			argc,
 				 char **		argv)
 {
-	int c;
+	init_helpers (argc, argv);
 
-#ifndef HAVE_PROGRAM_INVOCATION_NAME
-	{
-		unsigned int i;
-
-		for (i = strlen (argv[0]); i > 0; --i) {
-			if ('/' == argv[0][i - 1])
-				break;
-		}
-
-		program_invocation_short_name = &argv[0][i];
-	}
-#endif
-
-	setlocale (LC_ALL, "");
+	option_in_file_format = FILE_FORMAT_SLICED;
 
 	for (;;) {
 		int c;
@@ -1184,22 +805,6 @@ main				(int			argc,
 			option_decode_8302 ^= TRUE;
 			break;
 
-		case 'b':
-			option_dump_bin ^= TRUE;
-			break;
-
-		case 'c':
-			option_decode_caption ^= TRUE;
-			break;
-
-		case 'd':
-			option_idl_address = strtol (optarg, NULL, 10);
-			break;
-
-		case 'e':
-			option_dump_hex ^= TRUE;
-			break;
-
 		case 'a':
 			option_decode_ttx = TRUE;
 			option_decode_8301 = TRUE;
@@ -1212,16 +817,43 @@ main				(int			argc,
 			option_pfc_pgno = 0x1DF;
 			break;
 
+		case 'b':
+			option_dump_bin ^= TRUE;
+			break;
+
+		case 'c':
+			option_decode_caption ^= TRUE;
+			break;
+
+		case 'd':
+			assert (NULL != optarg);
+			option_idl_address = strtol (optarg, NULL, 0);
+			break;
+
+		case 'e':
+			option_dump_hex ^= TRUE;
+			break;
+
 		case 'h':
 			usage (stdout);
 			exit (EXIT_SUCCESS);
 
 		case 'i':
+			assert (NULL != optarg);
+			option_in_file_name = optarg;
+			break;
+
+		case 'j':
 			option_decode_idl ^= TRUE;
 			break;
 
 		case 'l':
-			option_idl_channel = strtol (optarg, NULL, 10);
+			assert (NULL != optarg);
+			option_idl_channel = strtol (optarg, NULL, 0);
+			break;
+
+		case 'm':
+			option_dump_time ^= TRUE;
 			break;
 
 		case 'n':
@@ -1229,15 +861,21 @@ main				(int			argc,
 			break;
 
 		case 'p':
+			assert (NULL != optarg);
 			option_pfc_pgno = strtol (optarg, NULL, 16);
 			break;
 
+		case 'q':
+			parse_option_quiet ();
+			break;
+
 		case 'r':
 			option_decode_vps_other ^= TRUE;
 			break;
 
 		case 's':
-			option_pfc_stream = strtol (optarg, NULL, 10);
+			assert (NULL != optarg);
+			option_pfc_stream = strtol (optarg, NULL, 0);
 			break;
 
 		case 't':
@@ -1256,12 +894,18 @@ main				(int			argc,
 			option_decode_xds ^= TRUE;
 			break;
 
+		case 'M':
+			assert (NULL != optarg);
+			option_metronome_tick = strtod (optarg, NULL);
+			break;
+
 		case 'P':
-			source_is_pes ^= TRUE;
+			option_in_file_format = FILE_FORMAT_DVB_PES;
 			break;
 
 		case 'T':
-			option_dump_time ^= TRUE;
+			option_in_ts_pid = parse_option_ts ();
+			option_in_file_format = FILE_FORMAT_DVB_TS;
 			break;
 
 		case 'V':
@@ -1274,10 +918,6 @@ main				(int			argc,
 		}
 	}
 
-	if (isatty (STDIN_FILENO))
-		error_exit (_("No VBI data on standard input."));
-
-#ifdef HAVE_VBI_PFC_DEMUX
 	if (0 != option_pfc_pgno) {
 		pfc = vbi_pfc_demux_new (option_pfc_pgno,
 					 option_pfc_stream,
@@ -1286,7 +926,6 @@ main				(int			argc,
 		if (NULL == pfc)
 			no_mem_exit ();
 	}
-#endif
 
 	if (0 != option_idl_channel) {
 		idl = vbi_idl_a_demux_new (option_idl_channel,
@@ -1304,31 +943,26 @@ main				(int			argc,
 			no_mem_exit ();
 	}
 
-	c = getchar ();
-	ungetc (c, stdin);
+	rst = read_stream_new (option_in_file_name,
+			       option_in_file_format,
+			       option_in_ts_pid,
+			       decode_frame);
 
-	if (0 == c || source_is_pes) {
-		dvb = vbi_dvb_pes_demux_new (/* callback */ NULL,
-					     /* user_data */ NULL);
-		if (NULL == dvb)
-			no_mem_exit ();
+	stream_loop (rst);
 
-		pes_mainloop ();
-	} else {
-#if 2 == VBI_VERSION_MINOR /* XXX port me */
-		open_sliced_read (stdin);
-#endif
-		old_mainloop ();
-	}
+	stream_delete (rst);
+	rst = NULL;
+
+	error_msg (_("End of stream."));
 
-	vbi_dvb_demux_delete (dvb);
-	vbi_idl_demux_delete (idl);
-#ifdef HAVE_VBI_PFC_DEMUX
-	vbi_pfc_demux_delete (pfc);
-#endif
 	vbi_xds_demux_delete (xds);
+	xds = NULL;
+
+	vbi_idl_demux_delete (idl);
+	idl = NULL;
+
+	vbi_pfc_demux_delete (pfc);
+	pfc = NULL;
 
 	exit (EXIT_SUCCESS);
-
-	return 0;
 }
