@@ -16,10 +16,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
-/* $Id: osc.c,v 1.23 2006/04/29 05:55:35 mschimek Exp $ */
+/* $Id: osc.c,v 1.35 2008/03/01 07:36:41 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -37,11 +38,14 @@
 #include <getopt.h>
 #endif
 
-#include "src/libzvbi.h"
-
-#define TEST 1
+#include "src/io.h"
+#include "src/decoder.h"
 #include "src/misc.h"
+#include "src/hamm.h"
+#include "src/io-sim.h"
 #include "src/raw_decoder.h"	/* _vbi_service_table[] */
+#include "src/proxy-msg.h"
+#include "src/proxy-client.h"
 
 #ifndef X_DISPLAY_MISSING
 
@@ -51,6 +55,7 @@
 
 vbi_capture *		cap;
 vbi_raw_decoder *	par;
+vbi_proxy_client *      pxc = NULL;
 int			src_w, src_h;
 vbi_sliced *		sliced;
 int			slines;
@@ -76,22 +81,6 @@ int			draw_row, draw_offset;
 int			draw_count = -1;
 int                     cur_x, cur_y;
 
-#include "sim.c"
-
-vbi_inline int
-vbi_printable			(int			c)
-{
-	if (c < 0)
-		return '?';
-
-	c &= 0x7F;
-
-	if (c < 0x20 || c >= 0x7F)
-		return '.';
-
-	return c;
-}
-
 extern void
 vbi_capture_set_log_fp		(vbi_capture *		capture,
 				 FILE *			fp);
@@ -119,10 +108,11 @@ decode_ttx(uint8_t *buf, int line)
 	magazine = packet_address & 7;
 	packet = packet_address >> 3;
 
-        text += sprintf(text, "pg %x%02d ln %03d >", magazine, packet, line);
+        text += snprintf(text, 255,
+			 "pg %x%02d ln %03d >", magazine, packet, line);
 
         for (j = 0; j < 42; j++) {
-	   char c = vbi_printable (buf[j]);
+	   char c = _vbi_to_ascii (buf[j]);
 	   
 	   *text = c;
 	   text++;
@@ -148,17 +138,17 @@ dump_pil(int pil)
 	min = pil & 0x3F;
 
 	if (pil == PIL(0, 15, 31, 63))
-		sprintf(text, " PDC: Timer-control (no PDC)\n");
+		snprintf(text, 255, " PDC: Timer-control (no PDC)\n");
 	else if (pil == PIL(0, 15, 30, 63))
-		sprintf(text, " PDC: Recording inhibit/terminate\n");
+		snprintf(text, 255, " PDC: Recording inhibit/terminate\n");
 	else if (pil == PIL(0, 15, 29, 63))
-		sprintf(text, " PDC: Interruption\n");
+		snprintf(text, 255, " PDC: Interruption\n");
 	else if (pil == PIL(0, 15, 28, 63))
-		sprintf(text, " PDC: Continue\n");
+		snprintf(text, 255, " PDC: Continue\n");
 	else if (pil == PIL(31, 15, 31, 63))
-		sprintf(text, " PDC: No time\n");
+		snprintf(text, 255, " PDC: No time\n");
 	else
-		sprintf(text, " PDC: %05x, 200X-%02d-%02d %02d:%02d\n",
+		snprintf(text, 255, " PDC: %05x, 200X-%02d-%02d %02d:%02d\n",
 			pil, mon, day, hour, min);
         return text;
 }
@@ -176,7 +166,7 @@ decode_vps(uint8_t *buf)
         text_start=text=malloc(255);
         memset(text, 0, 255);
         
-	text += sprintf(text, "VPS: ");
+	text += snprintf(text, 255, "VPS: ");
 
 	c = vbi_rev8 (buf[1]);
 
@@ -188,11 +178,11 @@ decode_vps(uint8_t *buf)
 
 	c &= 0x7F;
 
-	label[l] = vbi_printable (c);
+	label[l] = _vbi_to_ascii (c);
 
 	l = (l + 1) % 16;
 
-	text += sprintf(text, " 3-10: %02x %02x %02x %02x %02x %02x %02x %02x (\"%s\")\n",
+	text += snprintf(text, 250, " 3-10: %02x %02x %02x %02x %02x %02x %02x %02x (\"%s\")\n",
 		buf[0], buf[1], buf[2], buf[3], buf[4], buf[5], buf[6], buf[7], pr_label);
 
 	pcs = buf[2] >> 6;
@@ -206,9 +196,10 @@ decode_vps(uint8_t *buf)
 
 	pty = buf[12];
 
-	text += sprintf(text, " CNI: %04x PCS: %d PTY: %d ", cni, pcs, pty);
+	/* FIXME use real buffer size. */
+	text += snprintf(text, 100, " CNI: %04x PCS: %d PTY: %d ", cni, pcs, pty);
 
-	text += sprintf(text, " %s", dump_pil(pil));
+	text += snprintf(text, 50, " %s", dump_pil(pil));
    
         return(text_start);
 }
@@ -248,21 +239,33 @@ draw(unsigned char *raw)
 
 	XPutImage(display, window, gc, ximage,
 		draw_offset, 0, 0, 0, rem, src_h);
+//XSync(display, False);
 
 	XSetForeground(display, gc, 0);
+//XSync(display, False);
 
-	if (rem < dst_w)
+	if (rem < dst_w) {
+//		fprintf(stderr, "%u: %p %u %u  %u %u %u %u\n",__LINE__,
+//			display, window, gc,
+//			rem, 0, dst_w, src_h);
 		XFillRectangle(display, window, gc,
 			rem, 0, dst_w, src_h);
+//XSync(display, False);
+	}
 
 	if ((v = dst_h - src_h) <= 0)
 		return;
 
 	XSetForeground(display, gc, 0);
+//XSync(display, False);
+//		fprintf(stderr, "%u: %p %u %u  %u %u %u %u\n",__LINE__,
+//display, window, gc,0, src_h, dst_w, dst_h);
 	XFillRectangle(display, window, gc,
 		0, src_h, dst_w, dst_h);
+//XSync(display, False);
 
 	XSetForeground(display, gc, ~0);
+//XSync(display, False);
 
 	field = (draw_row >= par->count[0]);
 
@@ -333,6 +336,7 @@ draw(unsigned char *raw)
 				       " (%d)", sd);
 	}
 
+/*
         XSetForeground(display, gc, 0x00FFFF00);
         XFillRectangle(display, window, gc,
 		       0xc0-draw_offset, 
@@ -340,24 +344,36 @@ draw(unsigned char *raw)
         XFillRectangle(display, window, gc,
 		       0x19b-draw_offset,
 		       src_h, 1, dst_h);
+*/
         /* 50% grey */
         XSetForeground(display, gc, 0xAAAAAAAA);  
+//XSync(display, False);
         x=draw_offset;
         while (x<src_w && (x-draw_offset)<dst_w) {
+//		fprintf(stderr, "%u: %p %u %u  %u %u %u %u\n",__LINE__,
+//display, window, gc,
+//			  x-draw_offset, /* x,y, w,h */
+//			  src_h, 1, dst_h);
 	   XFillRectangle(display, window, gc,
 			  x-draw_offset, /* x,y, w,h */
 			  src_h, 1, dst_h);
+//XSync(display, False);
 	   x+=10;
 	}
         XSetForeground(display, gc, ~0);
+//XSync(display, False);
 
         xti.chars = buf;
 	xti.delta = 0;
 	xti.font = 0;
 
 	XDrawText(display, window, gc, 4, src_h + 12, &xti, 1);
-        xti.nchars = snprintf(buf, 255, "(%d, %3.0d)", cur_x+draw_offset, (1000*(dst_h-cur_y))/(dst_h-src_h));
+//XSync(display, False);
+        xti.nchars = snprintf(buf, 255, "(%d, %3d)", cur_x+draw_offset,
+	  (dst_h - cur_y) * 256 / v);
+//	  (1000*(dst_h-cur_y))/(dst_h-src_h));
         XDrawText(display, window, gc, 4, src_h + 24, &xti, 1);
+//XSync(display, False);
 
 	data = raw + draw_offset + draw_row * src_w;
 	h0 = dst_h - (data[0] * v) / 256;
@@ -369,6 +385,7 @@ draw(unsigned char *raw)
 		int h = dst_h - (data[i] * v) / 256;
 
 		XDrawLine(display, window, gc, i - 1, h0, i, h);
+//XSync(display, False);
 		h0 = h;
 	}
 }
@@ -393,6 +410,7 @@ xevent(void)
 
 			case 'q':
 			case 'c':
+			case XK_Escape:
 				quit = TRUE;
 				break;
 
@@ -566,13 +584,8 @@ mainloop(void)
 	for (quit = FALSE; !quit;) {
 		int r;
 
-		if (do_sim) {
-			read_sim(raw1, sliced, &slines, &timestamp);
-			r = 1;
-		} else {
-			r = vbi_capture_read(cap, raw1, sliced,
-					     &slines, &timestamp, &tv);
-		}
+		r = vbi_capture_read(cap, raw1, sliced,
+				     &slines, &timestamp, &tv);
 
 		switch (r) {
 		case -1:
@@ -586,7 +599,7 @@ mainloop(void)
 		case 0: 
 			fprintf(stderr, "VBI read timeout%s\n",
 				ignore_error ? " (ignored)" : "");
-			if (ignore_error)
+			if (ignore_error || (pxc != NULL))
 				continue;
 			else
 				exit(EXIT_FAILURE);
@@ -604,7 +617,7 @@ mainloop(void)
 	}
 }
 
-static const char short_options[] = "123cd:enpsv";
+static const char short_options[] = "1234cd:enpsv";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
@@ -618,6 +631,7 @@ long_options[] = {
 	{ "v4l",	no_argument,		NULL,		'1' },
 	{ "v4l2-read",	no_argument,		NULL,		'2' },
 	{ "v4l2-mmap",	no_argument,		NULL,		'3' },
+	{ "proxy",	no_argument,		NULL,		'4' },
 	{ "verbose",	no_argument,		NULL,		'v' },
 	{ 0, 0, 0, 0 }
 };
@@ -648,6 +662,7 @@ main(int argc, char **argv)
 			/* fall through */
 		case '1':
 		case '3':
+		case '4':
 			interface = c - '0';
 			break;
 		case 'c':
@@ -684,10 +699,12 @@ main(int argc, char **argv)
 	strict = 0;
 
 	if (do_sim) {
-		par = init_sim (scanning, services, !desync);
+		cap = vbi_capture_sim_new (scanning, &services,
+					   /* interlaced */ FALSE, !desync);
+		assert ((par = vbi_capture_parameters(cap)));
 	} else {
 		do {
-			if (1 != interface) {
+			if ((2 == interface) || (3 == interface)) {
 				cap = vbi_capture_v4l2k_new
 					(dev_name, /* fd */ -1,
 					 /* buffers */ 5, &services,
@@ -736,6 +753,24 @@ main(int argc, char **argv)
 				free (errstr);
 			}
 
+			if (interface == 4) {
+                                pxc = vbi_proxy_client_create(dev_name, "capture", 0,
+                                                                &errstr, !!verbose);
+                                if (pxc != NULL) {
+                                        /* strip non-raw services, else request for raw is masked out */
+                                        unsigned int sv = services & (VBI_SLICED_VBI_525 |
+                                                                      VBI_SLICED_VBI_625);
+                                        cap = vbi_capture_proxy_new(pxc, 5, 0, &sv,
+                                                                    strict, &errstr );
+                                        if (cap)
+                                                break;
+
+                                        fprintf (stderr, "Cannot capture vbi data "
+                                                 "through proxy:\n%s\n", errstr);
+                                }
+				fprintf (stderr, "Cannot initialize proxy\n%s\n", errstr);
+                        }
+
 			/* BSD interface */
 			if (1) {
 				cap = vbi_capture_bktr_new (dev_name,
